r"""
Global options

The :class:`GlobalOptions` class provides a generic mechanism for
setting and accessing **global** options for parents in one or several
related classes, typically for customizing the representation of their
elements. This class will eventually also support setting options on a
parent by parent basis.

These options should be "attached" to one or more classes as an options method.

.. SEEALSO::

    For good examples of :class:`GlobalOptions` in action see
    :obj:`sage.combinat.partition.Partitions.options` and
    :obj:`sage.combinat.tableau.Tableaux.options`.

.. _construction_section:

Construction of options classes
-------------------------------

The general setup for creating a set of global options is::

    sage: from sage.structure.global_options import GlobalOptions
    sage: class MyOptions(GlobalOptions):
    ....:     '''
    ....:     Nice options
    ....:
    ....:     @OPTIONS@
    ....:     '''
    ....:     NAME = 'option name'
    ....:     module = 'sage.some_module.some_file'
    ....:     option_class = 'name_of_class_controlled_by_options'
    ....:     first_option = dict(default='with_bells',
    ....:                       description='Changes the functionality of _repr_',
    ....:                       values=dict(with_bells='causes _repr_ to print with bells',
    ....:                                   with_whistles='causes _repr_ to print with whistles'),
    ....:                       alias=dict(bells='option1', whistles='option2'))
    ....:     # second_option = dict(...)
    ....:     # third_option = dict(...)

Note the syntax using the ``class`` keyword. However, because of some
metaclass magic, the resulting ``MyOptions`` object becomes an instance
of ``GlobalOptions`` instead of a subclass. So, despite the ``class``
syntax, ``MyOptions`` is not a class.

The options constructed by :class:`GlobalOptions` have to be explicitly
associated to the class that they control using the following arguments:

- ``NAME`` -- A descriptive name for the options class. This is
  optional; the default is the name of the constructed class.

- ``module`` -- The sage module containing the options class (optional)

- ``option_class`` -- The name of the options class. This is optional and
  defaults to ``NAME`` if not explicitly set.

It is only possible to pickle a :class:`GlobalOptions` class if the
corresponding module is specified *and* if the options are explicitly
attached to the corresponding class as a *options* method.

Each option is specified as a dictionary which describes the possible
values for the option and its documentation. The possible entries in this
dictionary are:

- ``alias`` -- Allows for several option values to do the same thing.

- ``alt_name`` -- An alternative name for this option.

- ``checker`` -- A validation function which returns whether a user
  supplied value is valid or not. This is typically useful for large
  lists of legal values such as :class:`~sage.rings.semirings.non_negative_integer_semiring.NN`.

- ``default`` -- Gives the default value for the option.

- ``description`` -- A one line description of the option.

- ``link_to`` -- Links this option to another one in another set of
  global options. This is used for example to allow
  :class:`Partitions` and :class:`Tableaux` to share the same
  ``convention`` option.

- ``setter`` -- A function which is called **after** the value of the
  option is changed.

- ``values`` -- A dictionary assigning each valid value for the option
  to a short description of what it does.

- ``case_sensitive`` -- (Default: ``True``) ``True`` or ``False`` depending on
  whether the values of the option are case sensitive.

For each option, either a complete list of possible values, via ``values``, or a
validation function, via ``checker``, must be given. The values can be quite
arbitrary, including user-defined functions which customize the default
behaviour of the classes such as the output of ``_repr_`` or :func:`latex`. See
:ref:`dispatcher` below, and :meth:`~GlobalOptions._dispatcher`, for more
information.

The documentation for the options is automatically constructed from
the docstring of the class by replacing the magic word ``@OPTIONS@``
with a description of each option.

The basic structure for defining a :class:`GlobalOptions` class is best
illustrated by an example::

    sage: from sage.structure.global_options import GlobalOptions
    sage: class Menu(object):
    ....:     class options(GlobalOptions):
    ....:         '''
    ....:         Fancy documentation
    ....:         -------------------
    ....:
    ....:         @OPTIONS@
    ....:
    ....:         The END!
    ....:         '''
    ....:         NAME = 'menu'
    ....:         entree = dict(default='soup',
    ....:                     description='The first course of a meal',
    ....:                     values=dict(soup='soup of the day', bread='oven baked'),
    ....:                     alias=dict(rye='bread'))
    ....:         appetizer = dict(alt_name='entree')
    ....:         main = dict(default='pizza', description='Main meal',
    ....:                   values=dict(pizza='thick crust', pasta='penne arrabiata'),
    ....:                   case_sensitive=False)
    ....:         dessert = dict(default='espresso', description='Dessert',
    ....:                      values=dict(espresso='life begins again',
    ....:                                  cake='waist begins again',
    ....:                                  cream='fluffy, white stuff'))
    ....:         tip = dict(default=10, description='Reward for good service',
    ....:                    checker = lambda tip: tip in range(0,20))
    sage: Menu.options
    Current options for menu
      - dessert: espresso
      - entree:  soup
      - main:    pizza
      - tip:     10

In the examples above, the options are constructed when the ``options``
object is created. However, it is also possible to construct the options
dynamically using the :meth:`GlobalOptions._add_to_options` methods.

For more details see :class:`GlobalOptions`.

Accessing and setting option values
-----------------------------------

All options and their values, when they are strings, are forced to be lower
case. The values of an options class can be set and accessed by calling the
class or by treating the class as an array.

Continuing the example from :ref:`construction_section`::

    sage: Menu.options
    Current options for menu
      - dessert: espresso
      - entree:  soup
      - main:    pizza
      - tip:     10
    sage: Menu.options.dessert
    espresso
    sage: Menu.options.dessert = 'cake'
    sage: Menu.options.dessert
    cake

Note that, provided there is no ambiguity, options and their values can be
abbreviated::

    sage: Menu.options('d')
    'cake'
    sage: Menu.options('m','t',des='esp', ent='sou')  # get and set several values at once
    ['pizza', 10]
    sage: Menu.options(t=15)
    sage: Menu.options('tip')
    15
    sage: Menu.options.tip
    15
    sage: Menu.options(e='s', m='Pi'); Menu.options()
    Current options for menu
      - dessert: cake
      - entree:  soup
      - main:    pizza
      - tip:     15
    sage: Menu.options(m='P')
    Traceback (most recent call last):
    ...
    ValueError: P is not a valid value for main in the options for menu


Setter functions
----------------

Each option of a :class:`GlobalOptions` can be equipped with an optional setter
function which is called **after** the value of the option is changed. In the
following example, setting the option 'add' changes the state of the class by
setting an attribute in this class using a :func:`classmethod`. Note that the
options object is inserted after the creation of the class in order to access
the :func:`classmethod` as ``A.setter``::

    sage: from sage.structure.global_options import GlobalOptions
    sage: class A(SageObject):
    ....:     state = 0
    ....:     @classmethod
    ....:     def setter(cls, option, val):
    ....:         cls.state += int(val)
    sage: class options(GlobalOptions):
    ....:     NAME = "A"
    ....:     add = dict(default=1,
    ....:                checker=lambda v: int(v)>0,
    ....:                description='An option with a setter',
    ....:                setter=A.setter)
    sage: A.options = options
    sage: A.options
    Current options for A
    - add: 1
    sage: a = A(); a.state
    1
    sage: a.options()
    Current options for A
    - add: 1
    sage: a.options(add=4)
    sage: a.state
    5
    sage: a.options()
    Current options for A
    - add: 4

Documentation for options
-------------------------

The documentation for a :class:`GlobalOptions` is automatically generated from
the supplied options. For example, the generated documentation for the options
``menu`` defined in :ref:`construction_section` is the following:

.. CODE-BLOCK:: text

    Fancy documentation
    -------------------

    OPTIONS:

    - ``appetizer`` -- alternative name for ``entree``
    - ``dessert`` -- (default: ``espresso``)
      Dessert

      - ``cake``     -- waist begins again
      - ``cream``    -- fluffy, white stuff
      - ``espresso`` -- life begins again

    - ``entree`` -- (default: ``soup``)
      The first course of a meal

      - ``bread`` -- oven baked
      - ``rye``   -- alias for ``bread``
      - ``soup``  -- soup of the day

    - ``main`` -- (default: ``pizza``)
      Main meal

      - ``pasta`` -- penne arrabiata
      - ``pizza`` -- thick crust

    - ``tip`` -- (default: ``10``)
      Reward for good service



    The END!

    See :class:`~sage.structure.global_options.GlobalOptions` for more features of these options.

In addition, help on each option, and its list of possible values, can be
obtained by (trying to) set the option equal to '?'::

    sage: Menu.options.dessert?                # not tested
    - ``dessert`` -- (default: ``espresso``)
      Dessert

      - ``cake``     -- waist begins again
      - ``cream``    -- fluffy, white stuff
      - ``espresso`` -- life begins again

.. _dispatcher:

Dispatchers
-----------

The whole idea of a :class:`GlobalOptions` class is that the options change the
default behaviour of the associated classes. This can be done either by simply
checking what the current value of the relevant option is. Another possibility
is to use the options class as a dispatcher to associated methods. To use the
dispatcher feature of a :class:`GlobalOptions` class it is necessary to implement
separate methods for each value of the option where the naming convention for
these methods is that they start with a common prefix and finish with the value
of the option.

If the value of a dispatchable option is set equal to a (user defined) function
then this function is called instead of a class method.

For example, the options ``MyOptions`` can be used to dispatch the ``_repr_``
method of the associated class ``MyClass`` as follows:

.. CODE-BLOCK:: python

    class MyClass(...):
        def _repr_(self):
            return self.options._dispatch(self,'_repr_','first_option')
        def _repr_with_bells(self):
            print('Bell!')
        def _repr_with_whistles(self):
            print('Whistles!')
    class MyOptions(GlobalOptions):
        ...

In this example, ``first_option`` is an option of ``MyOptions`` which takes
values ``bells``, ``whistles``, and so on. Note that it is necessary to make
``self``, which is an instance of ``MyClass``, an argument of the dispatcher
because :meth:`~GlobalOptions._dispatch()` is a method of :class:`GlobalOptions`
and not a method of ``MyClass``. Apart from ``MyOptions``, as it is a method of
this class, the arguments are the attached class (here ``MyClass``), the prefix
of the method of ``MyClass`` being dispatched, the option of ``MyOptions``
which controls the dispatching. All other arguments are passed through to the
corresponding methods of ``MyClass``. In general, a dispatcher is invoked as:

.. CODE-BLOCK:: python

    self.options._dispatch(self, dispatch_to, option, *args, **kargs)

Usually this will result in the method
``dispatch_to + '_' + MyOptions(options)`` of ``self`` being called with
arguments ``*args`` and ``**kargs`` (if ``dispatch_to[-1] == '_'`` then the
method ``dispatch_to + MyOptions(options)`` is called).

If ``MyOptions(options)`` is itself a function then the dispatcher will call
this function instead. In this way, it is possible to allow the user to
customise the default behaviour of this method. See
:meth:`~GlobalOptions._dispatch` for an example of how this can be achieved.

The dispatching capabilities of :class:`GlobalOptions` allows options to be
applied automatically without needing to parse different values of the option
(the cost is that there must be a method for each value). The dispatching
capabilities can also be used to make one option control several methods:

.. CODE-BLOCK:: python

    def __le__(self, other):
        return self.options._dispatch(self, '_le_','cmp', other)
    def __ge__(self, other):
        return self.options._dispatch(self, '_ge_','cmp', other)
    def _le_option_a(self, other):
        return ...
    def _ge_option_a(self, other):
        return ...
    def _le_option_b(self, other):
        return ...
    def _ge_option_b(self, other):
        return ...

See :meth:`~GlobalOptions._dispatch` for more details.

Doc testing
-----------

All of the options and their effects should be doc-tested. However, in order
not to break other tests, all options should be returned to their default state
at the end of each test. To make this easier, every :class:`GlobalOptions` class has
a :meth:`~GlobalOptions._reset()` method for doing exactly this.


Pickling
--------

Options classes can only be pickled if they are the options for some standard
sage class. In this case the class is specified using the arguments to
:class:`GlobalOptions`. For example
:meth:`~sage.combinat.partition.Partitions.options` is defined as:

.. CODE-BLOCK:: python

     class Partitions(UniqueRepresentation, Parent):
         ...
         class options(GlobalOptions):
             NAME = 'Partitions'
             module = 'sage.combinat.partition'
             ...

Here is an example to test the pickling of a :class:`GlobalOptions` instance::

    sage: TestSuite(Partitions.options).run()

TESTS:

Check that the old call syntax still works::

    sage: class Menu(object):
    ....:     options = GlobalOptions('menu',
    ....:         doc='Fancy documentation\n'+'-'*19, end_doc='The END!',
    ....:         entree=dict(default='soup',
    ....:                     description='The first course of a meal',
    ....:                     values=dict(soup='soup of the day', bread='oven baked'),
    ....:                     alias=dict(rye='bread')),
    ....:         appetizer=dict(alt_name='entree'),
    ....:         main=dict(default='pizza', description='Main meal',
    ....:                   values=dict(pizza='thick crust', pasta='penne arrabiata'),
    ....:                   case_sensitive=False),
    ....:         dessert=dict(default='espresso', description='Dessert',
    ....:                      values=dict(espresso='life begins again',
    ....:                                  cake='waist begins again',
    ....:                                  cream='fluffy, white stuff')),
    ....:         tip=dict(default=10, description='Reward for good service',
    ....:         checker=lambda tip: tip in range(0,20))
    ....:     )
    sage: Menu.options
    Current options for menu
      - dessert: espresso
      - entree:  soup
      - main:    pizza
      - tip:     10

We can have a ``name`` option::

    sage: class MyOptions(GlobalOptions):
    ....:     name = dict(default='alice', values={'alice': "A", 'bob': "B"})
    sage: MyOptions
    Current options for MyOptions
      - name: alice

Check that the ``name`` and ``NAME`` keywords are both supported with
this syntax::

    sage: GlobalOptions(name="menu")
    Current options for menu
    sage: GlobalOptions(NAME="menu")
    Current options for menu
    sage: GlobalOptions()
    Traceback (most recent call last):
    ...
    TypeError: GlobalOptions() is missing keyword argument 'name'

Test the documentation examples above::

    sage: print(Menu.options.__doc__)
    Fancy documentation
    -------------------
    <BLANKLINE>
    OPTIONS:
    <BLANKLINE>
    - ``appetizer`` -- alternative name for ``entree``
    - ``dessert`` -- (default: ``espresso``)
      Dessert
    <BLANKLINE>
      - ``cake``     -- waist begins again
      - ``cream``    -- fluffy, white stuff
      - ``espresso`` -- life begins again
    <BLANKLINE>
    - ``entree`` -- (default: ``soup``)
      The first course of a meal
    <BLANKLINE>
      - ``bread`` -- oven baked
      - ``rye``   -- alias for ``bread``
      - ``soup``  -- soup of the day
    <BLANKLINE>
    - ``main`` -- (default: ``pizza``)
      Main meal
    <BLANKLINE>
      - ``pasta`` -- penne arrabiata
      - ``pizza`` -- thick crust
    <BLANKLINE>
    - ``tip`` -- (default: ``10``)
      Reward for good service
    <BLANKLINE>
    <BLANKLINE>
    <BLANKLINE>
    <BLANKLINE>
    The END!
    See :class:`~sage.structure.global_options.GlobalOptions` for more features of these options.

::

    sage: print(Menu.options.dessert.__doc__)
    - ``dessert`` -- (default: ``espresso``)
      Dessert
    <BLANKLINE>
      - ``cake``     -- waist begins again
      - ``cream``    -- fluffy, white stuff
      - ``espresso`` -- life begins again
    <BLANKLINE>

AUTHORS:

- Andrew Mathas (2013): initial version
- Andrew Mathas (2016): overhaul making the options attributes, enabling
                        pickling and attaching the options to a class.
- Jeroen Demeyer (2017): use subclassing to create instances
"""

# ****************************************************************************
#       Copyright (C) 2013,2016 Andrew Mathas <andrew dot mathas at sydney dot edu dot au>
#       Copyright (C) 2017 Jeroen Demeyer <J.Demeyer@UGent.be>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


from importlib import import_module
from pickle import PicklingError
from textwrap import dedent

from sage.docs.instancedoc import instancedoc


class Option(object):
    r"""
    An option.

    Each option for an options class is an instance of this class which
    implements the magic that allows the options to the attributes of the
    options class that can be looked up, set and called.

    By way of example, this class implements the following functionality.

    EXAMPLES::

        sage: Partitions.options.display
        list
        sage: Partitions.options.display='compact'
        sage: Partitions.options.display('list')
        sage: Partitions.options._reset()

    TESTS::

        sage: TestSuite(Partitions.options.display).run()
    """
    __name__ = 'Option class'

    def __init__(self, options, name):
        r"""
        Initialise an option by settings its ``name``, "parent" option class
        ``options`` and doc-string.

        EXAMPLES::

            sage: type(Partitions.options.display)    # indirect doctest
            <class 'sage.structure.global_options.Option'>
        """
        self._name = name
        self._options = options
        self.__doc__ = options._doc[name]
        super(Option, self).__init__()

    def __repr__(self):
        r"""
        Return a string representation for this collection of options.

        EXAMPLES::

            sage: Partitions.options.display # indirect doctest
            list
        """
        # NOTE: we intentionally use str() instead of repr()
        return str(self._options[self._name])

    def __add__(self, other):
        r"""
        Return the object obtained by adding ``self`` and ``other``, where
        ``self`` behaves like its value.

        EXAMPLES::

            sage: Tableaux.options.convention +' is good'
            'English is good'
        """
        return self._options[self._name] + other

    def __radd__(self, other):
        r"""
        Return the object obtained by adding ``other`` and ``self``, where
        ``self`` behaves like its value.

        EXAMPLES::

            sage: 'Good '+Tableaux.options.convention
            'Good English'
        """
        return other + self._options[self._name]

    def __mul__(self, other):
        r"""
        Return the object obtained by adding ``self`` and ``other``, where
        ``self`` behaves like its value.

        EXAMPLES::

            sage: Tableaux.options.convention +' is good'
            'English is good'
        """
        return self._options[self._name] * other

    def __rmul__(self, other):
        r"""
        Return the object obtained by r-adding ``other`` and ``self``, where
        ``self`` behaves like its value.

        EXAMPLES::

            sage: 'Good '+Tableaux.options.convention
            'Good English'
        """
        return other * self._options[self._name]

    def __bool__(self):
        r"""
        Return the value of this option interpreted as a boolean.

        EXAMPLES::

            sage: RiggedConfigurations.options.half_width_boxes_type_B
            True
            sage: 'yes' if RiggedConfigurations.options.half_width_boxes_type_B else  'no'
            'yes'
            sage: RiggedConfigurations.options.half_width_boxes_type_B=False
            sage: 'yes' if RiggedConfigurations.options.half_width_boxes_type_B else  'no'
            'no'
            sage: RiggedConfigurations.options._reset()
        """
        return bool(self._options[self._name])

    # for the less sensibly named python 2 family
    __nonzero__ = __bool__

    def __call__(self, *args, **kwds):
        r"""
        Get or set value of the option ``self``.

        EXAMPLES::

            sage: Partitions.options.display() # indirect doctest
            'list'
            sage: Partitions.options.display('exp') # indirect doctest
            sage: Partitions.options.display() # indirect doctest
            'exp_low'
            sage: Partitions.options._reset()

        TESTS:

        Check that values can be set to ``None`` (:trac:`30763`)::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class config(GlobalOptions):
            ....:     size = dict(default=42,
            ....:                 description='integer or None',
            ....:                 checker=lambda val: val is None or val >= 0)
            sage: config.size()
            42
            sage: config.size(None)
            sage: config.size() is None
            True
            sage: config._reset()

        Check the deprecation::

            sage: config.size(value=None)
            doctest:...: DeprecationWarning: keyword argument "value" should be replaced by positional argument
            See https://trac.sagemath.org/30763 for details.
            sage: config.size() is None
            True
            sage: config.size(1, 2)
            Traceback (most recent call last):
            ...
            TypeError: option takes at most one argument "value"
            sage: config.size(unknown=3)
            Traceback (most recent call last):
            ...
            TypeError: option takes at most one argument "value"
            sage: config.size(4, value=5)
            Traceback (most recent call last):
            ...
            TypeError: option takes at most one argument "value"
        """
        if not args and not kwds:
            return self._options[self._name]
        if 'value' in kwds:
            from sage.misc.superseded import deprecation
            deprecation(30763, 'keyword argument "value" should be replaced '
                               'by positional argument')
            args += (kwds.pop('value'),)
        if len(args) > 1 or kwds:
            raise TypeError('option takes at most one argument "value"')
        self._options[self._name] = args[0]

    def __eq__(self, other):
        r"""
        Equality testing for an option in based on the value of the attribute.

        EXAMPLES::

            sage: Tableaux.options.convention
            English
            sage: Tableaux.options.convention == "English"
            True
            sage: Tableaux.options.convention == "French"
            False
        """
        return self._options[self._name] == other

    def __ne__(self, other):
        r"""
        Inequality testing for an option in based on the value of
        the attribute.

        EXAMPLES::

            sage: Tableaux.options.convention
            English
            sage: Tableaux.options.convention != "English"
            False
            sage: Tableaux.options.convention != "French"
            True
        """
        return self._options[self._name] != other

    def __hash__(self):
        r"""
        Return the hash of ``self``, which is the hash of the corresponding
        value.

        EXAMPLES::

            sage: hash(Tableaux.options.convention) == hash(Tableaux.options('convention'))
            True
        """
        return hash(self._options[self._name])

    def __str__(self):
        r"""
        Return the string representation of ``self``, which is the string of
        the corresponding value.

        EXAMPLES::

            sage: str(Tableaux.options.convention)
            'English'
        """
        return str(self._options[self._name])


class GlobalOptionsMetaMeta(type):
    def __call__(self, name, bases, dict):
        """
        Called when subclassing an instance of ``self``.

        Python translates ``class C(B): ...`` to
        ``meta = type(B); C = meta("C", (B,), ...)``.
        If we want to intercept this call ``meta(...)``, we need to
        customize ``__call__`` in the metaclass of ``meta``, which is
        this metametaclass.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: type(GlobalOptions)
            <class 'sage.structure.global_options.GlobalOptionsMeta'>
            sage: type(type(GlobalOptions))
            <class 'sage.structure.global_options.GlobalOptionsMetaMeta'>
            sage: class G(GlobalOptions): pass
            sage: type(G)
            <class 'sage.structure.global_options.GlobalOptions'>

        Since ``G`` is constructed using ``class`` syntax, the object
        gets a ``__module__`` attribute, different from the
        ``__module__`` attribute of its type (:class:`GlobalOptions`)::

            sage: G.__module__
            '__main__'
            sage: type(G).__module__
            'sage.structure.global_options'

        Multiple base classes are not allowed::

            sage: class G(GlobalOptions, object): pass
            Traceback (most recent call last):
            ...
            TypeError: GlobalOptions must be the only base class
        """
        # Allow only a single base class (which is GlobalOptions in
        # practice).
        # If we ever find a reasonable meaning for multiple base
        # classes, note that Python 2 and Python 3 have different
        # semantics for determining the metaclass when multiple base
        # classes are involved.
        #
        # Note: On Python 3 bases is empty if the class was declared
        # without any explicted bases:
        if not bases:
            bases = (object,)

        if len(bases) != 1:
            raise TypeError("GlobalOptions must be the only base class")

        base = bases[0]
        if not isinstance(base, self):
            # For the creation of the initial GlobalOptions class, fall
            # back to the usual class creation
            return self.__new__(self, name, bases, dict)

        # Create an instance of the base class (as opposed to an
        # instance of self, which would be typical for a metaclass)
        instance = base.__new__(base)

        # Split dict in options for instance.__init__ and attributes to
        # insert in the class __dict__
        kwds = {"NAME": name}
        for key, value in dict.items():
            if key.startswith("__"):
                instance.__dict__[key] = value
            else:
                kwds[key] = value

        instance.__init__(**kwds)
        return instance


class GlobalOptionsMeta(type, metaclass=GlobalOptionsMetaMeta):
    """
    Metaclass for :class:`GlobalOptions`

    This class is itself an instance of :class:`GlobalOptionsMetaMeta`,
    which implements the subclass magic.
    """


@instancedoc
class GlobalOptions(metaclass=GlobalOptionsMeta):
    r"""
    The :class:`GlobalOptions` class is a generic class for setting and
    accessing global options for Sage objects.

    While it is possible to create instances of :class:`GlobalOptions`
    the usual way, the recommended syntax is to subclass from
    ``GlobalOptions``. Thanks to some metaclass magic, this actually
    creates an instance of ``GlobalOptions`` instead of a subclass.

    INPUT (as "attributes" of the class):

    - ``NAME`` -- specifies a name for the options class (optional;
      default: class name)

    - ``module`` -- gives the module that contains the associated options class

    - ``option_class`` -- gives the name of the associated module class
      (default: ``NAME``)

    - option = ``dict(...)`` -- dictionary specifying an option

    The options are specified by keyword arguments with their values
    being a dictionary which describes the option. The
    allowed/expected keys in the dictionary are:

    - ``alias`` -- defines alias/synonym for option values
    - ``alt_name`` -- alternative name for an option
    - ``checker`` -- a function for checking whether a particular value for
      the option is valid
    - ``default`` -- the default value of the option
    - ``description`` -- documentation string
    - ``link_to`` -- links to an option for this set of options to an
      option in another :class:`GlobalOptions`
    - ``setter`` -- a function (class method) which is called whenever this
      option changes
    - ``values`` -- a dictionary of the legal values for this option (this
      automatically defines the corresponding ``checker``); this dictionary
      gives the possible options, as keys, together with a brief description
      of them
    - ``case_sensitive`` -- (default: ``True``) ``True`` or ``False``
      depending on whether the values of the option are case sensitive

    Options and their values can be abbreviated provided that this
    abbreviation is a prefix of a unique option.

    EXAMPLES::

        sage: from sage.structure.global_options import GlobalOptions
        sage: class Menu(object):
        ....:     class options(GlobalOptions):
        ....:         '''
        ....:         Fancy documentation
        ....:         -------------------
        ....:
        ....:         @OPTIONS@
        ....:
        ....:         End of Fancy documentation
        ....:         '''
        ....:         NAME = 'menu'
        ....:         entree = dict(default='soup',
        ....:                     description='The first course of a meal',
        ....:                     values=dict(soup='soup of the day', bread='oven baked'),
        ....:                     alias=dict(rye='bread'))
        ....:         appetizer = dict(alt_name='entree')
        ....:         main = dict(default='pizza', description='Main meal',
        ....:                   values=dict(pizza='thick crust', pasta='penne arrabiata'),
        ....:                   case_sensitive=False)
        ....:         dessert = dict(default='espresso', description='Dessert',
        ....:                      values=dict(espresso='life begins again',
        ....:                                  cake='waist begins again',
        ....:                                  cream='fluffy white stuff'))
        ....:         tip = dict(default=10, description='Reward for good service',
        ....:                  checker=lambda tip: tip in range(0,20))
        sage: Menu.options
        Current options for menu
          - dessert: espresso
          - entree:  soup
          - main:    pizza
          - tip:     10
        sage: Menu.options(entree='s')         # unambiguous abbreviations are allowed
        sage: Menu.options(t=15)
        sage: (Menu.options['tip'], Menu.options('t'))
        (15, 15)
        sage: Menu.options()
        Current options for menu
          - dessert: espresso
          - entree:  soup
          - main:    pizza
          - tip:     15
        sage: Menu.options._reset(); Menu.options()
        Current options for menu
          - dessert: espresso
          - entree:  soup
          - main:    pizza
          - tip:     10
        sage: Menu.options.tip=40
        Traceback (most recent call last):
        ...
        ValueError: 40 is not a valid value for tip in the options for menu
        sage: Menu.options(m='p')           # ambiguous abbreviations are not allowed
        Traceback (most recent call last):
        ...
        ValueError: p is not a valid value for main in the options for menu

    The documentation for the options class is automatically generated from the
    information which specifies the options:

    .. CODE-BLOCK:: text

        Fancy documentation
        -------------------

        OPTIONS:

        - dessert:  (default: espresso)
          Dessert

          - ``cake``     -- waist begins again
          - ``cream``    -- fluffy white stuff
          - ``espresso`` -- life begins again

        - entree:  (default: soup)
          The first course of a meal

          - ``bread`` -- oven baked
          - ``rye``   -- alias for bread
          - ``soup``  -- soup of the day

        - main:  (default: pizza)
          Main meal

          - ``pasta`` -- penne arrabiata
          - ``pizza`` -- thick crust

        - tip:  (default: 10)
          Reward for good service

        End of Fancy documentation

        See :class:`~sage.structure.global_options.GlobalOptions` for more features of these options.

    The possible values for an individual option can be obtained by
    (trying to) set it equal to '?'::

        sage: Menu.options(des='?')
        - ``dessert`` -- (default: ``espresso``)
          Dessert
        <BLANKLINE>
          - ``cake``     -- waist begins again
          - ``cream``    -- fluffy white stuff
          - ``espresso`` -- life begins again
        <BLANKLINE>
        Current value: espresso
    """
    __name__ = 'options'

    def __init__(self, NAME=None, module='', option_class='', doc='', end_doc='', **options):
        r"""
        Initialize ``self``.

        TESTS::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class menu(GlobalOptions):
            ....:     entree = dict(default='soup',
            ....:                 description='The first course of a meal',
            ....:                 values=dict(soup='soup of the day', bread='oven baked'),
            ....:                 alias=dict(rye='bread'))
            ....:     appetizer = dict(alt_name='entree')
            ....:     main = dict(default='pizza', description='Main meal',
            ....:               values=dict(pizza='thick crust', pasta='penne arrabiata'),
            ....:               case_sensitive=False)
            ....:     dessert = dict(default='espresso', description='Dessert',
            ....:                  values=dict(espresso='life begins again',
            ....:                              cake='waist begins again',
            ....:                              cream='fluffy white stuff'))
            ....:     tip = dict(default=10, description='Reward for good service',
            ....:              checker=lambda tip: tip in range(0,20))
            sage: menu._name  # Default name is class name
            'menu'
            sage: class specials(GlobalOptions):
            ....:     entree = dict(link_to=(menu, 'entree'))
            ....:     main_specials = dict(default='salmon', description='main course specials',
            ....:                   values=dict(salmon='a fish', crab='Sebastian'))
            sage: specials['entree'] = 'rye'
            sage: menu['entree']
            'bread'

            sage: class alias_test(GlobalOptions):
            ....:       "Test aliases with case sensitivity"
            ....:       test_opt = dict(default="Upper",
            ....:           description = 'Starts with an uppercase',
            ....:           values = dict(Upper="Starts with uppercase",
            ....:                         lower="only lowercase"),
            ....:           case_sensitive = False,
            ....:           alias = dict(UpperAlias="Upper", lower_alias="lower"))
            sage: alias_test['test_opt'] = 'Lower_Alias'
            sage: alias_test['test_opt']
            'lower'
            sage: alias_test['test_opt'] = 'upperalias'
            sage: alias_test['test_opt']
            'Upper'
        """
        if NAME is None:
            # Require a "name" keyword in **options
            try:
                NAME = options.pop("name")
            except KeyError:
                raise TypeError("GlobalOptions() is missing keyword argument 'name'")
        self._name = NAME

        # initialise the various dictionaries used by GlobalOptions
        self._alias = {}           # a dictionary of alias for the values of some options
        self._alt_names = {}       # a dictionary of alternative names for some options
        self._case_sensitive = {}  # a dictionary of booleans indicating to check case sensitivity
        self._checker = {}         # a dictionary of validity checkers for each option
        self.__default_value = {}  # a dictionary of the default options
        self._display_values = {}  # a dictionary of the output of the values
        self._doc = {}             # a dictionary of doc strings, forced by the linked options
        self._legal_values = {}    # a dictionary of lists of the legal values for each option
        self._linked_value = {}    # a dictionary of linked to other global options as (link, linked_option)
        self._setter = {}          # a dictionary of the list of setters
        self._value = {}           # a dictionary of the current options
        for option in options:
            self._add_option(option, options[option])

        self._options_module = module
        self._option_class = option_class or self._name

        # If the instance dict has a __doc__ attribute, use that as
        # docstring.
        try:
            self._docstring = dedent(self.__dict__.pop("__doc__"))
        except KeyError:
            self._docstring = "@OPTIONS@"

        # Finally, we build the doc string for the options
        # First we strip common white space off the front of doc and end_doc
        if doc:
            self._docstring = dedent(doc) + "\n\n" + self._docstring

        if end_doc:
            self._docstring = self._docstring + "\n\n" + dedent(end_doc)

        # Add docstring footer
        self._docstring += "\nSee :class:`~sage.structure.global_options.GlobalOptions` for more features of these options."

    def __repr__(self):
        r"""
        Return a string representation for this collection of options.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            sage: FoodOptions
            Current options for daily meal
              - drink: water
              - food:  apple
        """
        options = list(self._value) + list(self._linked_value)
        for x in self._alt_names:
            options.remove(x)
        if not options:
            return 'Current options for {}'.format(self._name)

        options.sort()
        width = 1 + max(len(option) for option in options)
        txt = '\n'.join('  - {:{}} {}'.format(option + ':', width, self[option])
                        for option in options)
        return 'Current options for {}\n{}'.format(self._name, txt)

    def __call__(self, *get_value, **set_value):
        r"""
        Get or set value of the option ``option``.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            ....:     beverage = dict(alt_name='drink')
            sage: FoodOptions()
            Current options for daily meal
              - drink: water
              - food:  apple
            sage: FoodOptions('food')
            'apple'
            sage: FoodOptions(food="pair"); FoodOptions()
            Current options for daily meal
              - drink: water
              - food:  pair
            sage: FoodOptions('beverage')
            'water'
            sage: FoodOptions(food="apple", drink="coffee"); FoodOptions()
            Current options for daily meal
              - drink: coffee
              - food:  apple
        """
        if not get_value and not set_value:
            return self

        if get_value:
            # use __getitem__ to return these options
            if len(get_value) == 1:
                return self[get_value[0]]
            else:
                return [self[option] for option in get_value]

        # use __setitem__ to set these options
        if set_value:
            for option in set_value:
                self[option] = set_value[option]

    def __getitem__(self, option):
        r"""
        Return the current value of the option ``option``.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            sage: FoodOptions['drink']
            'water'
            sage: FoodOptions['d']
            'water'
        """
        option = self._match_option(option)
        if option in self._linked_value:
            link, linked_opt = self._linked_value[option]
            return link[linked_opt]
        elif option in self._value:
            if option in self._display_values:
                return self._display_values[option][self._value[option]]
            return self._value[option]

    def __setitem__(self, option, value):
        r"""
        The ``__setitem__`` method is used to change the current values of the
        options. It also checks that the supplied options are valid and changes
        any alias to its generic value.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            sage: FoodOptions['drink']='coffee'; FoodOptions()
            Current options for daily meal
              - drink: coffee
              - food:  apple
            sage: FoodOptions(drink='w'); FoodOptions()
            Current options for daily meal
              - drink: water
              - food:  apple
            sage: FoodOptions(drink='?')
            - ``drink`` -- (default: ``water``)
            <BLANKLINE>
              - ``coffee`` -- a lifestyle
              - ``water``  -- a drink
            <BLANKLINE>
            Current value: water
        """
        option = self._match_option(option)
        if not callable(value):
            value = self._match_value(option, value)

        if value == '?':  # return help
            print('%s\nCurrent value: %s' % (self._doc[option], self[option]))
            return      # we do not want to call the setter below

        elif option in self._linked_value:
            link, linked_opt = self._linked_value[option]
            link[linked_opt] = value

        else:
            self._value[option] = value

        if option in self._setter:
            # if a setter function exists then call it with the associated
            # class, option and value
            self._setter[option](option, value)

    def _instancedoc_(self):
        r"""
        Return the docstring for the options class ``self``.

        EXAMPLES::

            sage: print(Partitions.options.__doc__)
            <BLANKLINE>
            Sets and displays the global options for elements of the partition,
            skew partition, and partition tuple classes.  If no parameters are
            set, then the function returns a copy of the options dictionary.
            <BLANKLINE>
            The ``options`` to partitions can be accessed as the method
            :obj:`Partitions.options` of :class:`Partitions` and
            related parent classes.
            <BLANKLINE>
            <BLANKLINE>
            OPTIONS:
            <BLANKLINE>
            - ``convention`` -- (default: ``English``)
              Sets the convention used for displaying tableaux and partitions
            <BLANKLINE>
              - ``English`` -- use the English convention
              - ``French``  -- use the French convention
            ...

        TESTS::

            sage: from sage.structure.global_options import GlobalOptions
            sage: print(GlobalOptions.__doc__)
            <BLANKLINE>
                The :class:`GlobalOptions` class is a generic class for setting...
        """
        options = 'OPTIONS:\n\n{}\n\n'.format('\n'.join(self._doc[opt] for opt in sorted(self._doc)))
        return self._docstring.replace("@OPTIONS@", options)

    def __setattr__(self, name, value=None):
        r"""
        Set the attribute ``name`` of the option class self equal to
        ``value``, if the attribute ``name`` exists.

        As the attributes of an option class are the actual options we need
        to be able to "trap" invalid options in a sensible way. We do this
        by sending any "non-standard" to :meth:`__setitem__` for processing.

        EXAMPLES::

            sage: Partitions.options.display = 'exp'
            sage: Partitions.options.dispplay = 'list'
            Traceback (most recent call last):
            ...
            ValueError: dispplay is not an option for Partitions
            sage: Partitions.options._reset()
        """
        # Underscore, and "special", attributes are set using type.__setattr__
        if name[0] == '_' or name in ['reset', 'dispatch', 'default_value']:
            return super(GlobalOptions, self).__setattr__(name, value)

        # General case: redirect to __setitem__
        self[name] = value

    def __setstate__(self, state):
        r"""
        This is a custom :meth:`__setstate__` method for unpickling instances of
        the :class:`GlobalOptions` class.

        The :meth:`__getstate__` method returns a dictionary with an
        `options_class` key which identifies the "parent" class for the options.
        This is then used to unpickle the options class.

        EXAMPLES::

            sage: Partitions.options()
            Current options for Partitions
              - convention:        English
              - diagram_str:       *
              - display:           list
              - latex:             young_diagram
              - latex_diagram_str: \ast
            sage: Partitions.options.convention="French"
            sage: pickle = dumps(Partitions.options)
            sage: Partitions.options._reset()        # reset options
            sage: loads(pickle)                      # indirect doctest
            Current options for Partitions
              - convention:        French
              - diagram_str:       *
              - display:           list
              - latex:             young_diagram
              - latex_diagram_str: \ast
            sage: Partitions.options._reset()
        """
        # open the options for the corresponding "parent" and copy all of
        # the data from its options class into unpickle
        options_class = getattr(import_module(state['options_module']), state['option_class'])
        unpickle = options_class.options
        state.pop('option_class')
        state.pop('options_module')
        for setting in unpickle.__dict__:
            self.__dict__[setting] = unpickle.__dict__[setting]

        # reset the options in `self` to their defaults
        self._reset()
        # apply the options stored in state
        for opt in state:
            self[opt] = state[opt]
        options_class.options = self

    def __getstate__(self):
        r"""
        Return a dictionary that can be used to pickle an instance of a
        :class:`GlobalOptions` class.

        This is called by :func:`pickle_GlobalOptions`.

        Typically instances of :class:`GlobalOptions` are
        complicated to create, so they do no pickle. If the options are
        associated to  "parent" class then it is possible to create the
        default version of the class and then add the non-default
        settings on top of this. This method returns a dictionary of the
        non-default options that can then be used to unpickle an
        instance of the option using :func:`unpickle_GlobalOptions`.

        EXAMPLES::

            sage: Partitions.options._reset()
            sage: Partitions.options.__getstate__()
             {'convention': 'English',
             'option_class': 'Partitions',
             'options_module': 'sage.combinat.partition'}
        """

        # options classes can be pickled only if they are the options for an
        # associated "parent" class that lives in self._module

        if not self._options_module:
            pickleable = False
        else:
            opt_mod = import_module(self._options_module)
            pickleable = hasattr(opt_mod, self._name) and hasattr(getattr(opt_mod, self._name), 'options')

        if not pickleable:
            raise PicklingError('%s cannot be pickled because it is not associated with a class' % self)

        pickle = {'option_class': self._option_class, 'options_module': self._options_module}
        for opt in self._value:
            if opt not in self._alt_names and self[opt] != self.__default_value[opt]:
                pickle[opt] = self[opt]
        for opt in self._linked_value:
            link, linked_opt = self._linked_value[opt]
            if opt not in self._alt_names and link[opt] != link.__default_value[opt]:
                pickle[opt] = self[opt]
        return pickle

    def __eq__(self, other):
        r"""
        Two options classes are equal if they return the same :meth:`__getstate__.

        EXAMPLES::

            sage: Partitions.options == PartitionsGreatestLE.options # indirect doctest
            True
            sage: Partitions.options == Tableaux.options
            False
        """
        return self.__getstate__() == other.__getstate__()

    def _add_option(self, option, specifications):
        r"""
        Add an option.

        INPUT:

        - ``option`` -- a string
        - ``specifications`` -- a dictionary

        .. SEEALSO::

            :class:`GlobalOptions` for a description of the required
            ``specifications``.
        """
        if not isinstance(specifications, dict):
            raise TypeError("expected dict as specification of %r, got %r" % (option, specifications))

        doc = {}  # will be used to build the doc string
        self._case_sensitive[option] = True    # ``True`` by default
        self._legal_values[option] = []
        for spec in sorted(specifications):   # NB: options processed alphabetically!
            if spec == 'alias':
                self._alias[option] = specifications[spec]
                self._legal_values[option] += list(specifications[spec])
                for opt in specifications[spec]:
                    doc[opt] = 'alias for ``%s``' % specifications[spec][opt]
            elif spec == 'alt_name':
                self._alt_names[option] = specifications[spec]
                self._linked_value[option] = (self, specifications[spec])
                doc = '- ``%s`` -- alternative name for ``%s``' % (option, specifications[spec].lower())
            elif spec == 'case_sensitive':
                if not specifications[spec]:
                    for opt in self._legal_values:
                        self._display_values[option] = {val.lower(): val for val in self._legal_values[option]}
                        self._legal_values[option] = [val.lower() for val in self._legal_values[option]]
                    if option in self._alias:
                        self._alias[option] = {k.lower(): v.lower()
                                               for k, v in self._alias[option].items()}
                self._case_sensitive[option] = bool(specifications[spec])
            elif spec == 'checker':
                if not callable(specifications[spec]):
                    raise ValueError('the checker for %s must be callable' % option)
                self._checker[option] = specifications[spec]
            elif spec == 'default':
                self.__default_value[option] = specifications[spec]
            elif spec == 'link_to':
                if (isinstance(specifications[spec], tuple) and
                        len(specifications[spec]) == 2 and
                        isinstance(specifications[spec][0], GlobalOptions)):
                    link, linked_opt = specifications['link_to']  # for sanity
                    if linked_opt in link._value:
                        self._linked_value[option] = specifications['link_to']
                        link, linked_opt = specifications['link_to']  # for sanity
                        doc = link._doc[linked_opt]
                    elif linked_opt in link._linked_value:
                        self._linked_value[option] = link._linked_value[linked_opt]
                        doc = link._doc[linked_opt]
                    else:
                        raise ValueError("could not find link to {1} in {0}".format(*specifications[spec]))
                else:
                    raise ValueError("linked options must be specified as a string: 'linked_option' or a tuple: (link,linked_option)")
            elif spec == 'setter':
                if callable(specifications[spec]):
                    self._setter[option] = specifications[spec]
                else:
                    raise ValueError('the setter for %s must be a function' % option)
            elif spec == 'values':
                for val in specifications[spec]:
                    doc[val] = specifications[spec][val]
                doc.update(specifications[spec])
                if self._case_sensitive[option]:
                    self._legal_values[option] += list(specifications[spec])
                    self._display_values[option] = {val: val for val in specifications[spec]}
                else:
                    self._legal_values[option] += [val.lower() for val in specifications[spec]]
                    self._display_values[option] = {val.lower(): val for val in specifications[spec]}
            elif spec != 'description':
                raise ValueError('Initialization error in Global options for %s: %s not recognized!' % (self._name, spec))

        # now build the doc string for this option
        if doc == {} and 'description' not in specifications:
            raise ValueError('no documentation specified for %s in the options for %s' % (option, self._name))

        # first a necessary hack to initialise the option in self._doc because __setitem__ calls _match_option
        self._doc[option] = ''
        if option in self._linked_value:
            self._doc[option] = doc
        else:
            width = max(len(v) for v in doc) + 4 if doc != {} else 4
            if len(doc) > 0:
                self._doc[option] = '- ``{}`` -- (default: ``{}``)\n{}\n{}\n'.format(
                    option, self._default_value(option),
                    '  %s\n' % specifications['description'] if 'description' in specifications else '',
                    '\n'.join('  - {:{}} -- {}'.format('``' + val + '``', width, doc[val])
                              for val in sorted(doc)))
            else:
                self._doc[option] = '- ``{}`` -- (default: ``{}``)\n{}'.format(
                    option, self._default_value(option),
                    '  %s\n' % specifications['description'] if 'description' in specifications else '')

        # sanity check for non-linked options
        if option not in self._linked_value:
            if 'default' not in specifications:
                raise ValueError('a default value for %s must be given' % option)

            if not (option in self._checker or option in self._legal_values):
                raise ValueError('a value checker or a list of valid values for %s must be given' % option)

            # finally, set, check and process the default value using  __setitem__
            self[option] = self.__default_value[option]
            self.__default_value[option] = self._value[option]  # in case the default is an alias

        # Build getters and setters for this option. As we have
        # overridden __setattr__, we call object.__setattr__ directly
        super(GlobalOptions, self).__setattr__(option, Option(self, option))

    def _match_option(self, option):
        r"""
        Check whether ``option`` is the name of an option of ``self``, or a
        prefix thereof.

        INPUT:

        - ``option`` -- a string

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            sage: FoodOptions('food') # indirect doctest
            'apple'
            sage: FoodOptions('f')
            'apple'
        """
        if option in self._doc:
            return option

        # a lower case version of the option
        loption = option.lower()

        # as it is not an option try and match it with a prefix to an option,
        # without checking case using the fact that the keys of self._doc is a
        # list of the options, both normal and linked
        matches = [opt for opt in self._doc if opt.lower().startswith(loption)]
        if matches and all(m.startswith(matches[0]) for m in matches):
            return matches[0]
        elif len(matches) > 1:
            # as there is more than one match check case as well
            matches = [mat for mat in matches if mat.startswith(option)]
            if matches and all(m.startswith(matches[0]) for m in matches):
                return matches[0]
            else:
                raise ValueError('%s is an ambiguous option for %s' % (option, self._name))

        # if we are still here this is not a good option!
        raise ValueError('%s is not an option for %s' % (option, self._name))

    def _match_value(self, option, value):
        r"""
        Check whether ``value`` is a valid value for ``option``.

        INPUT:

        - ``option`` -- a string: the name of an option, or prefix thereof
        - ``value``  -- a value or ``'?'``

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', wine='a lifestyle'))
            sage: FoodOptions(f='a') # indirect doctest
            sage: FoodOptions('f')
            'apple'
            sage: FoodOptions(d='wi'); FoodOptions('f')
            'apple'
            sage: FoodOptions(d='w')
            Traceback (most recent call last):
            ...
            ValueError: w is not a valid value for drink in the options for daily meal
        """
        if value == "?":
            return value   # help on this value

        if option in self._linked_value:
            link, linked_opt = self._linked_value[option]
            return link._match_value(linked_opt, value)

        orig_value = value

        # convert to proper case if it is a string
        if isinstance(value, str) and not self._case_sensitive[option]:
            value = value.lower()

        if option in self._legal_values:
            if value in self._legal_values[option]:
                if option in self._alias and value in self._alias[option]:
                    return self._alias[option][value]
                return value

            # as it is not a value try and match it with a prefix of a value
            matches = [val for val in self._legal_values[option] if val.startswith(value)]
            if matches and all(m.startswith(matches[0]) for m in matches):
                val = matches[0]
                if option in self._alias and val in self._alias[option]:
                    return self._alias[option][val]
                return val

        if option in self._checker and self._checker[option](value):
            return value

        # if we are still here this is not a good value!

        # replace any value alias with its "real" value
        if option in self._alias and value in self._alias[option]:
            orig_value = self._alias[option][value]
        raise ValueError('%s is not a valid value for %s in the options for %s' % (orig_value, option, self._name))

    def _default_value(self, option):
        r"""
        Return the default value of the option.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class FoodOptions(GlobalOptions):
            ....:     NAME = 'daily meal'
            ....:     food = dict(default='apple', values=dict(apple='a fruit', pair='of what?'))
            ....:     drink = dict(default='water', values=dict(water='a drink', coffee='a lifestyle'))
            sage: FoodOptions._default_value('food')
            'apple'
        """
        option = self._match_option(option)
        if option in self.__default_value:
            return self.__default_value[option]
        else:
            link, linked_opt = self._linked_value[option]
            return link._default_value(linked_opt)

    def _dispatch(self, obj, dispatch_to, option, *args, **kargs):
        r"""
        .. TODO:: title

        The *dispatchable* options are options which dispatch related methods of
        the corresponding class - or user defined methods which are passed to
        :class:`GlobalOptions`. The format for specifying a dispatchable option
        is to include ``dispatch_to = <option name>`` in the specifications for
        the options and then to add the options to the (element) class. Each
        option is then assumed to be a method of the element class with a name
        of the form ``<option name> + '_' + <current vale for this option'``.
        These options are called by the element class via::

            return self.options._dispatch(self, dispatch_to, option, *args, **kargs)

        Note that the argument ``self`` is necessary here because the
        dispatcher is a method of the options class and not of
        ``self``. The value of the option can also be set to a
        user-defined function, with arguments ``self`` and ``option``;
        in this case the user's function is called instead.

        EXAMPLES:

        Here is a contrived example::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class DelimitedListOptions(GlobalOptions):
            ....:           delim=dict(default='b', values={'b':'brackets', 'p':'parentheses'})
            sage: class DelimitedList(SageObject):
            ....:    options = DelimitedListOptions
            ....:    def __init__(self, L):
            ....:        self._list = L
            ....:    def _repr_b(self): return '[%s]' % ','.join('%s'%i for i in self._list)
            ....:    def _repr_p(self): return '(%s)' % ','.join('%s'%i for i in self._list)
            ....:    def _repr_(self): return self.options._dispatch(self, '_repr_','delim')
            sage: dlist = DelimitedList([1,2,3]); dlist
            [1,2,3]
            sage: dlist.options.delim='p'; dlist
            (1,2,3)
            sage: dlist.options(delim=lambda self: '<%s>' % ','.join('%s'%i for i in self._list)); dlist
            <1,2,3>
        """
        if callable(self._value[option]):
            try:
                return self._value[option](obj, *args, **kargs)
            except TypeError:
                raise ValueError('the user defined dispatcher function failed!')
        else:
            if dispatch_to[-1] == '_':
                dispatch_to = dispatch_to[:-1]
            dispatch = getattr(obj, dispatch_to + '_' + self._value[option])
            return dispatch(*args, **kargs)

        raise ValueError('%s is not a dispatchable option!' % option)

    def _reset(self, option=None):
        r"""
        Reset options to their default value.

        INPUT:

        - ``option`` -- (Default: ``None``) The name of an option as a string
          or ``None``. If ``option`` is specified only this option is reset to
          its default value; otherwise all options are reset.

        EXAMPLES::

            sage: from sage.structure.global_options import GlobalOptions
            sage: class Meal(object):
            ....:     class options(GlobalOptions):
            ....:         NAME = 'daily meal'
            ....:         food = dict(default='bread', values=dict(bread='rye bread', salmon='a fish'))
            ....:         drink = dict(default='water',values=dict(water='essential for life', wine='essential'))
            sage: Meal.options.food='salmon'; Meal.options
            Current options for daily meal
              - drink: water
              - food:  salmon
            sage: Meal.options._reset('drink'); Meal.options()
            Current options for daily meal
              - drink: water
              - food:  salmon
            sage: Meal.options._reset(); Meal.options()
            Current options for daily meal
              - drink: water
              - food:  bread
        """
        if option is None:
            for option in self.__default_value:
                self._value[option] = self.__default_value[option]
                if not self._case_sensitive[option] and isinstance(self._value[option], str):
                    self._value[option] = self._value[option].lower()
            for option in self._linked_value:
                link, linked_opt = self._linked_value[option]
                link._reset(linked_opt)
        else:
            option = self._match_option(option)
            if option in self.__default_value:
                self._value[option] = self.__default_value[option]
                if not self._case_sensitive[option] and isinstance(self._value[option], str):
                    self._value[option] = self._value[option].lower()
            elif option in self._linked_value:
                link, linked_opt = self._linked_value[option]
                link._reset(linked_opt)



r"""
Semirngs
"""
#*****************************************************************************
#  Copyright (C) 2010 Nicolas Borie <nicolas.borie@math.u-psud.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.category_with_axiom import CategoryWithAxiom
from .magmas_and_additive_magmas import MagmasAndAdditiveMagmas

class Semirings(CategoryWithAxiom):
    """
    The category of semirings.

    A semiring `(S,+,*)` is similar to a ring, but without the
    requirement that each element must have an additive inverse. In
    other words, it is a combination of a commutative additive monoid
    `(S,+)` and a multiplicative monoid `(S,*)`, where `*` distributes
    over `+`.

    .. SEEALSO::

        :wikipedia:`Semiring`

    EXAMPLES::

        sage: Semirings()
        Category of semirings
        sage: Semirings().super_categories()
        [Category of associative additive commutative additive associative additive unital distributive magmas and additive magmas,
         Category of monoids]

        sage: sorted(Semirings().axioms())
        ['AdditiveAssociative', 'AdditiveCommutative', 'AdditiveUnital', 'Associative', 'Distributive', 'Unital']

        sage: Semirings() is (CommutativeAdditiveMonoids() & Monoids()).Distributive()
        True

        sage: Semirings().AdditiveInverse()
        Category of rings


    TESTS::

        sage: TestSuite(Semirings()).run()
    """
    _base_category_class_and_axiom = (MagmasAndAdditiveMagmas.Distributive.AdditiveAssociative.AdditiveCommutative.AdditiveUnital.Associative, "Unital")




"""
Coerce actions
"""

#*****************************************************************************
#     Copyright (C) 2007 Robert Bradshaw <robertwb@math.washington.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import operator

from cpython.int cimport *
from cpython.number cimport *
from cysignals.signals cimport sig_check

from .coerce cimport coercion_model
from .element cimport parent, Element, ModuleElement
from .parent cimport Parent
from .coerce_exceptions import CoercionException
from sage.categories.action cimport InverseAction, PrecomposedAction
from sage.arith.long cimport integer_check_long


cdef _record_exception():
    coercion_model._record_exception()

cdef inline an_element(R):
    if isinstance(R, Parent):
        return R.an_element()
    else:
        for x in ([(1, 2)], "abc", 10.5, 10):
            try:
                return R(x)
            except Exception:
                pass


# In the code below, I take the convention that g is acting on a.

cdef class GenericAction(Action):

    cdef _codomain

    def __init__(self, Parent G, S, is_left, bint check=True):
        """
        TESTS:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: M = MatrixSpace(ZZ,2)
            sage: sage.structure.coerce_actions.ActedUponAction(M, Cusps, True)
            Left action by Full MatrixSpace of 2 by 2 dense matrices over Integer Ring on Set P^1(QQ) of all cusps

            sage: Z6 = Zmod(6)
            sage: sage.structure.coerce_actions.GenericAction(QQ, Z6, True)
            Traceback (most recent call last):
            ...
            NotImplementedError: action for <class 'sage.structure.coerce_actions.GenericAction'> not implemented

        This will break if we tried to use it::

            sage: sage.structure.coerce_actions.GenericAction(QQ, Z6, True, check=False)
            Left action by Rational Field on Ring of integers modulo 6

        """
        Action.__init__(self, G, S, is_left, operator.mul)
        if check:
            res = self.act(G.an_element(), S.an_element())
            if res is None:
                raise CoercionException
            _codomain = parent(res)

    def codomain(self):
        """
        Returns the "codomain" of this action, i.e. the Parent in which the
        result elements live. Typically, this should be the same as the
        acted upon set.

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains, for
        otherwise they could be garbage collected, giving rise to random
        errors (see :trac:`18157`). ::


            sage: M = MatrixSpace(ZZ,2)
            sage: A = sage.structure.coerce_actions.ActedUponAction(M, Cusps, True)
            sage: A.codomain()
            Set P^1(QQ) of all cusps

            sage: S3 = SymmetricGroup(3)                                                # optional - sage.groups
            sage: QQxyz = QQ['x,y,z']                                                   # optional - sage.groups
            sage: A = sage.structure.coerce_actions.ActOnAction(S3, QQxyz, False)       # optional - sage.groups
            sage: A.codomain()                                                          # optional - sage.groups
            Multivariate Polynomial Ring in x, y, z over Rational Field

        """
        if self._codomain is None:
            self._codomain = parent(self.act(an_element(self.G),
                                               an_element(self.underlying_set())))
        return self._codomain


cdef class ActOnAction(GenericAction):
    """
    Class for actions defined via the _act_on_ method.
    """
    cpdef _act_(self, g, x):
        """
        TESTS::

            sage: G = SymmetricGroup(3)                                                 # optional - sage.groups
            sage: R.<x,y,z> = QQ[]                                                      # optional - sage.groups
            sage: A = sage.structure.coerce_actions.ActOnAction(G, R, False)            # optional - sage.groups
            sage: A(x^2 + y - z, G((1,2)))                                              # optional - sage.groups
            y^2 + x - z
            sage: A(x+2*y+3*z, G((1,3,2)))                                              # optional - sage.groups
            2*x + 3*y + z

            sage: type(A)                                                               # optional - sage.groups
            <... 'sage.structure.coerce_actions.ActOnAction'>
        """
        return (<Element>g)._act_on_(x, self._is_left)


cdef class ActedUponAction(GenericAction):
    """
    Class for actions defined via the _acted_upon_ method.
    """
    cpdef _act_(self, g, x):
        """
        TESTS::

            sage: M = MatrixSpace(ZZ,2)
            sage: A = sage.structure.coerce_actions.ActedUponAction(M, Cusps, True)
            sage: A.act(matrix(ZZ, 2, [1,0,2,-1]), Cusp(1,2))
            Infinity
            sage: A(matrix(ZZ, 2, [1,0,2,-1]), Cusp(1,2))
            Infinity

            sage: type(A)
            <... 'sage.structure.coerce_actions.ActedUponAction'>
        """
        return (<Element>x)._acted_upon_(g, not self._is_left)


def detect_element_action(Parent X, Y, bint X_on_left, X_el=None, Y_el=None):
    r"""
    Return an action of X on Y as defined by elements of X, if any.

    EXAMPLES:

    Note that coerce actions should only be used inside of the coercion
    model. For this test, we need to strongly reference the domains,
    for otherwise they could be garbage collected, giving rise to
    random errors (see :trac:`18157`). ::

        sage: from sage.structure.coerce_actions import detect_element_action
        sage: ZZx = ZZ['x']
        sage: M = MatrixSpace(ZZ,2)
        sage: detect_element_action(ZZx, ZZ, False)
        Left scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
        sage: detect_element_action(ZZx, QQ, True)
        Right scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
        sage: detect_element_action(Cusps, M, False)
        Left action by Full MatrixSpace of 2 by 2 dense matrices over Integer Ring on Set P^1(QQ) of all cusps
        sage: detect_element_action(Cusps, M, True),
        (None,)
        sage: detect_element_action(ZZ, QQ, True),
        (None,)

    TESTS:

    This test checks that the issue in :trac:`7718` has been fixed::

        sage: class MyParent(Parent):
        ....:     def an_element(self):
        ....:         pass
        ....:
        sage: A = MyParent()
        sage: detect_element_action(A, ZZ, True)
        Traceback (most recent call last):
        ...
        RuntimeError: an_element() for <__main__.MyParent object at ...> returned None
    """
    cdef Element x

    # sample elements x and y
    if X_el is None or (parent(X_el) is not X):
        x = an_element(X)
    else:
        x = X_el
    if x is None:
        raise RuntimeError("an_element() for %s returned None" % X)
    if Y_el is None or (parent(Y_el) is not Y):
        y = an_element(Y)
    else:
        y = Y_el
    if y is None:
        if isinstance(Y, Parent):
            raise RuntimeError("an_element() for %s returned None" % Y)
        else:
            return # don't know how to make elements of this type...

    # element x defining _lmul_ or _rmul_
    if isinstance(x, ModuleElement) and isinstance(y, Element):
        try:
            return (RightModuleAction if X_on_left else LeftModuleAction)(Y, X, y, x)
        except CoercionException as msg:
            _record_exception()

    # element x defining _act_on_
    try:
        if x._act_on_(y, X_on_left) is not None:
            return ActOnAction(X, Y, X_on_left, False)
    except CoercionException:
        _record_exception()

    # element x defining _acted_upon_
    if isinstance(Y, Parent):
        try:
            if x._acted_upon_(y, X_on_left) is not None:
                return ActedUponAction(Y, X, not X_on_left, False)
        except CoercionException:
            _record_exception()


cdef class ModuleAction(Action):
    """
    Module action.

    .. SEEALSO::

        This is an abstract class, one must actually instantiate a
        :class:`LeftModuleAction` or a :class:`RightModuleAction`.

    INPUT:

    - ``G`` -- the actor, an instance of :class:`~sage.structure.parent.Parent`.
    - ``S`` -- the object that is acted upon.
    - ``g`` -- optional, an element of ``G``.
    - ``a`` -- optional, an element of ``S``.
    - ``check`` -- if True (default), then there will be no consistency tests
      performed on sample elements.

    NOTE:

    By default, the sample elements of ``S`` and ``G`` are obtained from
    :meth:`~sage.structure.parent.Parent.an_element`, which relies on the
    implementation of an ``_an_element_()`` method. This is not always
    available. But usually, the action is only needed when one already
    *has* two elements. Hence, by :trac:`14249`, the coercion model will
    pass these two elements to the :class:`ModuleAction` constructor.

    The actual action is implemented by the ``_rmul_`` or ``_lmul_``
    function on its elements. We must, however, be very particular about
    what we feed into these functions, because they operate under the
    assumption that the inputs lie exactly in the base ring and may
    segfault otherwise. Thus we handle all possible base extensions
    manually here.

    """
    def __init__(self, G, S, g=None, a=None, check=True):
        """
        This creates an action of an element of a module by an element of its
        base ring. The simplest example to keep in mind is R acting on the
        polynomial ring R[x].

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::


            sage: from sage.structure.coerce_actions import LeftModuleAction
            sage: ZZx = ZZ['x']
            sage: QQx = QQ['x']
            sage: QQy = QQ['y']
            sage: ZZxy = ZZ['x']['y']
            sage: LeftModuleAction(ZZ, ZZx)
            Left scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
            sage: LeftModuleAction(ZZ, QQx)
            Left scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Rational Field
            sage: LeftModuleAction(QQ, ZZx)
            Left scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            sage: LeftModuleAction(QQ, ZZxy)
            Left scalar multiplication by Rational Field on Univariate Polynomial Ring in y over Univariate Polynomial Ring in x over Integer Ring

        The following tests against a problem that was relevant during work on
        :trac:`9944`::

            sage: R.<x> = PolynomialRing(ZZ)
            sage: S.<x> = PolynomialRing(ZZ, sparse=True)
            sage: 1/R.0
            1/x
            sage: 1/S.0
            1/x

        """
        Action.__init__(self, G, S, not isinstance(self, RightModuleAction), operator.mul)
        if not isinstance(G, Parent):
            # only let Parents act
            raise TypeError("Actor must be a parent.")
        base = S.base()
        if base is S or base is None:
            # The right thing to do is a normal multiplication
            raise CoercionException("Best viewed as standard multiplication")
        # Objects are implemented with the assumption that
        # _lmul_/_rmul_ are given an element of the base ring
        if G is not base:
            # first we try the easy case of coercing G to the base ring of S
            self.connecting = base._internal_coerce_map_from(G)
            if self.connecting is None:
                # otherwise, we try and find a base extension
                from sage.categories.pushout import pushout
                # this may raise a type error, which we propagate
                self.extended_base = pushout(G, S)
                # make sure the pushout actually gave a correct base extension of S
                if self.extended_base.base() != pushout(G, base):
                    raise CoercionException("Actor must be coercible into base.")
                else:
                    self.connecting = self.extended_base.base()._internal_coerce_map_from(G)
                    if self.connecting is None:
                        # this may happen if G is, say, int rather than a parent
                        # TODO: let python types be valid actions
                        raise CoercionException("Missing connecting morphism")

        # Don't waste time if our connecting morphisms happen to be the identity.
        if self.connecting is not None and self.connecting.codomain() is G:
            self.connecting = None

        if self.extended_base is not None and self.extended_base is S:
            self.extended_base = None

        # At this point, we can assert it is safe to call _Xmul_
        the_ring = G if self.connecting is None else self.connecting.codomain()
        the_set = S if self.extended_base is None else self.extended_base
        assert the_ring is the_set.base(), "BUG in coercion model\n    Apparently there are two versions of\n        %s\n    in the cache."%the_ring

        if not check:
            return
        if g is None:
            g = G.an_element()
        if parent(g) is not G:
            raise CoercionException("The parent of %s is not %s but %s"%(g,G,parent(g)))
        if a is None:
            a = S.an_element()
        if parent(a) is not S:
            raise CoercionException("The parent of %s is not %s but %s"%(a,S,parent(a)))
        if not isinstance(g, Element) or not isinstance(a, ModuleElement):
            raise CoercionException("not an Element acting on a ModuleElement")
        res = self.act(g, a)
        if parent(res) is not the_set:
            # In particular we will raise an error if res is None
            raise CoercionException("Result is None or has wrong parent.")

    def __reduce__(self):
        """
        Used in pickling.

        TESTS:

        Check that this action can be pickled (:trac:`29031`)::

            sage: A = ZZ['x'].get_action(QQ, self_on_left=False, op=operator.mul)
            sage: loads(dumps(A)) is not None
            True
        """
        return (type(self), (self.G, self.underlying_set()))

    def _repr_name_(self):
        """
        The default name of this action type, which is has a sane default.

        EXAMPLES:

        Note that coerce actions should only be used inside of the
        coercion model. For this test, we need to strongly reference the
        domains, for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import LeftModuleAction, RightModuleAction
            sage: ZZx = ZZ['x']
            sage: A = LeftModuleAction(ZZ, ZZx); A
            Left scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
            sage: A._repr_name_()
            'scalar multiplication'

            sage: GF5 = GF(5)
            sage: GF5t = GF5[['t']]
            sage: RightModuleAction(GF5, GF5t)
            Right scalar multiplication by Finite Field of size 5 on Power Series Ring in t over Finite Field of size 5

        """
        return "scalar multiplication"

    def codomain(self):
        """
        The codomain of self, which may or may not be equal to the domain.

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import LeftModuleAction
            sage: ZZxyz = ZZ['x,y,z']
            sage: A = LeftModuleAction(QQ, ZZxyz)
            sage: A.codomain()
            Multivariate Polynomial Ring in x, y, z over Rational Field
        """
        if self.extended_base is not None:
            return self.extended_base
        return self.underlying_set()

    def domain(self):
        """
        The domain of self, which is the module that is being acted on.

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import LeftModuleAction
            sage: ZZxyz = ZZ['x,y,z']
            sage: A = LeftModuleAction(QQ, ZZxyz)
            sage: A.domain()
            Multivariate Polynomial Ring in x, y, z over Integer Ring
        """
        return self.underlying_set()

    def __invert__(self):
        """
        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains, for
        otherwise they could be garbage collected, giving rise to random
        errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import RightModuleAction

            sage: ZZx = ZZ['x']
            sage: x = ZZx.gen()
            sage: QQx = QQ['x']
            sage: A = ~RightModuleAction(QQ, QQx); A
            Right inverse action by Rational Field on Univariate Polynomial Ring in x over Rational Field
            sage: A(x, 2)
            1/2*x

            sage: A = ~RightModuleAction(QQ, ZZx); A
            Right inverse action by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            sage: A.codomain()
            Univariate Polynomial Ring in x over Rational Field
            sage: A(x, 2)
            1/2*x

            sage: A = ~RightModuleAction(ZZ, ZZx); A
            Right inverse action by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Rational Field
            sage: A.codomain()
            Univariate Polynomial Ring in x over Rational Field
            sage: A(x, 2)
            1/2*x

            sage: GF5x = GF(5)['x']
            sage: A = ~RightModuleAction(ZZ, GF5x); A
            Right inverse action by Finite Field of size 5 on Univariate Polynomial Ring in x over Finite Field of size 5
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Finite Field of size 5
            sage: A(x, 2)
            3*x

            sage: GF5xy = GF5x['y']
            sage: A = ~RightModuleAction(ZZ, GF5xy); A
            Right inverse action by Finite Field of size 5 on Univariate Polynomial Ring in y over Univariate Polynomial Ring in x over Finite Field of size 5
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Finite Field of size 5

            sage: ZZy = ZZ['y']
            sage: ZZxyzw = ZZx['y']['z']['w']
            sage: A = ~RightModuleAction(ZZy, ZZxyzw); A
            Right inverse action by Fraction Field of Univariate Polynomial Ring in y
                over Univariate Polynomial Ring in x
                over Integer Ring
            on Univariate Polynomial Ring in w
                over Univariate Polynomial Ring in z
                over Univariate Polynomial Ring in y
                over Univariate Polynomial Ring in x
                over Integer Ring
            with precomposition on right by Conversion via FractionFieldElement map:
              From: Univariate Polynomial Ring in y over Integer Ring
              To:   Fraction Field of Univariate Polynomial Ring in y over Univariate Polynomial Ring in x over Integer Ring

        TESTS:

        See :trac:`19521`::

            sage: Q.<y> = SR.subring(no_variables=True)[[]]
            sage: (y / 1).parent()
            Power Series Ring in y over Symbolic Constants Subring
            sage: R.<x> = SR.subring(no_variables=True)[]
            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.explain(x, 1, operator.truediv)
            Action discovered.
                Right inverse action by Symbolic Constants Subring on Univariate Polynomial Ring in x over Symbolic Constants Subring
                with precomposition on right by Conversion via _symbolic_ method map:
                  From: Integer Ring
                  To:   Symbolic Constants Subring
            Result lives in Univariate Polynomial Ring in x over Symbolic Constants Subring
            Univariate Polynomial Ring in x over Symbolic Constants Subring
        """
        K = self.G._pseudo_fraction_field()
        if K is self.G:
            return InverseAction(self)
        else:
            # Try to find a suitable ring between G and R in which to compute
            # the inverse.
            from sage.categories.pushout import construction_tower, pushout
            R = self.G if self.connecting is None else self.connecting.codomain()
            K = pushout(self.G._pseudo_fraction_field(), R)
            if K is None:
                K = R._pseudo_fraction_field()

            # Suppose we have parents of a construction tower
            #   A -> B -> C <- D <- E -> F <- G -> H,
            # where the arrows specify the direction of coercion (i.e.
            # A -> B means A coerces to B). Note that B = FA(A), C = FB(B), ...
            # As we want to find a "smallest" parent with some properties,
            # we need the order
            #   A, B, E, D, C, G, F, H
            # for our search. Thus the elements connected with a <- have to
            # be reversed. See code below.
            tower = []
            reversed_part = []
            for Fi, Ri in reversed(construction_tower(K)):
                if Fi is not None and Fi.coercion_reversed:
                    reversed_part.append((Fi, Ri))
                else:
                    tower.append((Fi, Ri))
                    if reversed_part:
                        tower += reversed(reversed_part)
                        reversed_part = []
            assert(not reversed_part)
            for _, Ri in tower:
                if not Ri.has_coerce_map_from(self.G):
                    continue
                Ki = Ri._pseudo_fraction_field()
                if Ki is Ri:
                    K = Ki
                    break
            module_action = type(self)(K, self.codomain())
            connecting = K.coerce_map_from(self.G)
            left, right = (connecting, None) if self._is_left else (None, connecting)
            return PrecomposedAction(InverseAction(module_action), left, right)


cdef class LeftModuleAction(ModuleAction):
    cpdef _act_(self, g, a):
        """
        A left module action is an action that takes the ring element as the
        first argument (the left side) and the module element as the second
        argument (the right side).

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import LeftModuleAction
            sage: R.<x> = QQ['x']
            sage: A = LeftModuleAction(ZZ, R)
            sage: A(5, x+1)
            5*x + 5
            sage: R.<x> = ZZ['x']
            sage: A = LeftModuleAction(QQ, R)
            sage: A(1/2, x+1)
            1/2*x + 1/2
            sage: A(1/2, x+1)
            1/2*x + 1/2
        """
        # The way we are called, we know for sure that a is a
        # ModuleElement and that g is an Element.
        # If we change a or g, we need to explicitly check this,
        # which explains the <?> casts below.
        if self.connecting is not None:
            g = <Element?>self.connecting._call_(g)
        if self.extended_base is not None:
            a = <ModuleElement?>self.extended_base(a)
        return (<ModuleElement>a)._rmul_(<Element>g)  # g * a


cdef class RightModuleAction(ModuleAction):
    cpdef _act_(self, g, a):
        """
        A right module action is an action that takes the module element as the
        first argument (the left side) and the ring element as the second
        argument (the right side).

        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the domains,
        for otherwise they could be garbage collected, giving rise to
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import RightModuleAction
            sage: R.<x> = QQ['x']
            sage: A = RightModuleAction(ZZ, R)
            sage: A(x+5, 2)
            2*x + 10
            sage: A(x+5, 2)
            2*x + 10
        """
        # The way we are called, we know for sure that a is a
        # ModuleElement and that g is an Element.
        # If we change a or g, we need to explicitly check this,
        # which explains the <?> casts below.
        if self.connecting is not None:
            g = <Element?>self.connecting._call_(g)
        if self.extended_base is not None:
            a = <ModuleElement?>self.extended_base(a)
        return (<ModuleElement>a)._lmul_(<Element>g)  # a * g


cdef class IntegerAction(Action):
    """
    Abstract base class representing some action by integers on
    something. Here, "integer" is defined loosely in the "duck typing"
    sense.

    INPUT:

    - ``Z`` -- a type or parent representing integers

    For the other arguments, see :class:`Action`.

    .. NOTE::

        This class is used internally in Sage's coercion model. Outside
        of the coercion model, special precautions are needed to prevent
        domains of the action from being garbage collected.
    """
    def __init__(self, Z, S, is_left, op):
        if isinstance(Z, type):
            from sage.sets.pythonclass import Set_PythonType
            Z = Set_PythonType(Z)
        super().__init__(Z, S, is_left, op)

    def __reduce__(self):
        """
        Used in pickling.

        TESTS:

        Check that this action can be pickled (:trac:`29031`)::

            sage: from sage.structure.coerce_actions import IntegerMulAction
            sage: act = IntegerMulAction(ZZ, CDF)
            sage: loads(dumps(act)) is not None
            True
        """
        # All base classes must take the signature
        #   (Z, S, is_left)
        return (type(self), (self.G, self.underlying_set(), self._is_left))

    def __invert__(self):
        """
        EXAMPLES::

            sage: from sage.structure.coerce_actions import IntegerMulAction
            sage: act = IntegerMulAction(ZZ, CDF)
            sage: ~act
            Traceback (most recent call last):
            ...
            TypeError: actions by ZZ cannot be inverted
        """
        raise TypeError("actions by ZZ cannot be inverted")


cdef class IntegerMulAction(IntegerAction):
    r"""
    Implement the action `n \cdot a = a + a + ... + a` via repeated
    doubling.

    Both addition and negation must be defined on the set `M`.

    INPUT:

    - ``Z`` -- a type or parent representing integers

    - ``M`` -- a ``ZZ``-module

    - ``m`` -- (optional) an element of ``M``

    EXAMPLES::

        sage: from sage.structure.coerce_actions import IntegerMulAction
        sage: R.<x> = QQ['x']
        sage: act = IntegerMulAction(ZZ, R)
        sage: act(5, x)
        5*x
        sage: act(0, x)
        0
        sage: act(-3, x-1)
        -3*x + 3
    """
    def __init__(self, Z, M, is_left=True, m=None):
        if m is None:
            m = M.an_element()
        test = m + (-m)  # make sure addition and negation is allowed
        super().__init__(Z, M, is_left, operator.mul)

    cpdef _act_(self, nn, a):
        """
        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the field
        ``GF(101)``, for otherwise it could be garbage collected, giving rise
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import IntegerMulAction
            sage: GF101 = GF(101)
            sage: act = IntegerMulAction(ZZ, GF101)
            sage: act(3, 9)
            27
            sage: act(3^689, 9)
            42
            sage: 3^689 * mod(9, 101)
            42

        TESTS:

        Use round off error to verify this is doing actual repeated addition
        instead of just multiplying::

            sage: act = IntegerMulAction(ZZ, RR)
            sage: act(49, 1/49) == 49*RR(1/49)
            False

        This used to hang before :trac:`17844`::

            sage: E = EllipticCurve(GF(5), [4,0])
            sage: P = E.random_element()
            sage: (-2^63)*P
            (0 : 1 : 0)

        Check that large multiplications can be interrupted::

            sage: alarm(0.5); (2^(10^6)) * P
            Traceback (most recent call last):
            ...
            AlarmInterrupt

        Verify that cysignals correctly detects that the above
        exception has been handled::

            sage: from cysignals.tests import print_sig_occurred
            sage: print_sig_occurred()
            No current exception
        """
        cdef int err = 0
        cdef long n_long

        if integer_check_long(nn, &n_long, &err) and not err:
            return fast_mul_long(a, n_long)

        return fast_mul(a, nn)

    def _repr_name_(self):
        """
        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the field
        ``GF(5)``, for otherwise it could be garbage collected, giving rise
        random errors (see :trac:`18157`). ::

            sage: from sage.structure.coerce_actions import IntegerMulAction
            sage: GF5 = GF(5)
            sage: IntegerMulAction(ZZ, GF5)
            Left Integer Multiplication by Integer Ring on Finite Field of size 5
        """
        return "Integer Multiplication"


cdef class IntegerPowAction(IntegerAction):
    r"""
    The right action ``a ^ n = a * a * ... * a`` where `n` is an
    integer.

    The action is implemented using the ``_pow_int`` method on elements.

    INPUT:

    - ``Z`` -- a type or parent representing integers

    - ``M`` -- a parent whose elements implement ``_pow_int``

    - ``m`` -- (optional) an element of ``M``

    EXAMPLES::

        sage: from sage.structure.coerce_actions import IntegerPowAction
        sage: R.<x> = LaurentSeriesRing(QQ)
        sage: act = IntegerPowAction(ZZ, R)
        sage: act(x, 5)
        x^5
        sage: act(x, -2)
        x^-2
        sage: act(x, int(5))
        x^5

    TESTS::

        sage: IntegerPowAction(ZZ, R, True)
        Traceback (most recent call last):
        ...
        ValueError: powering must be a right action
        sage: IntegerPowAction(ZZ, QQ^3)
        Traceback (most recent call last):
        ...
        TypeError: no integer powering action defined on Vector space of dimension 3 over Rational Field

    ::

        sage: var('x,y')
        (x, y)
        sage: RDF('-2.3')^(x+y^3+sin(x))
        (-2.3)^(y^3 + x + sin(x))
        sage: RDF('-2.3')^x
        (-2.3)^x
    """
    def __init__(self, Z, M, is_left=False, m=None):
        if is_left:
            raise ValueError("powering must be a right action")
        if m is None:
            m = M.an_element()
        try:
            # Check that there is a _pow_int() method
            m._pow_int
        except AttributeError:
            raise TypeError(f"no integer powering action defined on {M}")
        super().__init__(Z, M, False, operator.pow)

    cpdef _act_(self, n, a):
        """
        EXAMPLES:

        Note that coerce actions should only be used inside of the coercion
        model. For this test, we need to strongly reference the field
        ``GF(101)``::

            sage: from sage.structure.coerce_actions import IntegerPowAction
            sage: GF101 = GF(101)
            sage: act = IntegerPowAction(ZZ, GF101)
            sage: act(3, 100)
            1
            sage: act(3, -1)
            34
            sage: act(3, 1000000000000000000000000000000000000000000001)
            3
        """
        cdef Element e = <Element>a
        cdef long value = 0
        cdef int err
        integer_check_long(n, &value, &err)
        if not err:
            return e._pow_long(value)
        return e._pow_int(n)

    def _repr_name_(self):
        """
        EXAMPLES::

            sage: from sage.structure.coerce_actions import IntegerPowAction
            sage: IntegerPowAction(ZZ, QQ)
            Right Integer Powering by Integer Ring on Rational Field
        """
        return "Integer Powering"


cdef inline fast_mul(a, n):
    if n < 0:
        n = -n
        a = -a
    pow2a = a
    while n & 1 == 0:
        sig_check()
        pow2a += pow2a
        n = n >> 1
    sum = pow2a
    n = n >> 1
    while n != 0:
        sig_check()
        pow2a += pow2a
        if n & 1:
            sum += pow2a
        n = n >> 1
    return sum

cdef inline fast_mul_long(a, long s):
    # It's important to change the signed s to an unsigned n,
    # since -LONG_MIN = LONG_MIN.  See Trac #17844.
    cdef unsigned long n
    if s < 0:
        n = -s
        a = -a
    else:
        n = s
    if n < 4:
        if n == 0:
            p = parent(a)
            try:
                return p.zero()
            except AttributeError:
                return p(0)
        elif n == 1: return a
        elif n == 2: return a+a
        elif n == 3: return a+a+a
    pow2a = a
    while n & 1 == 0:
        pow2a += pow2a
        n = n >> 1
    sum = pow2a
    n = n >> 1
    while n != 0:
        pow2a += pow2a
        if n & 1:
            sum += pow2a
        n = n >> 1
    return sum



"""
Except from the `random` module from Python 2.7.14 used solely for consistency
in the doctest suite for random results that depend on the Python PRNG.

It should be made clear that the actual random data from the PRNG is consistent
between Python versions, but the difference lies in higher-level methods such
as `random.randint`.

See :trac:`24508`
"""

# The following code was copied from
# https://github.com/python/cpython/blob/9b7ae96b98238627751e2f508ba5d600363555ef/Lib/random.py
# and was modified slightly for Python 3 compatibility.
#
# It has following copyrights; for the full Python license
# agreement see COPYING.txt at the root of Sage sources:
#
# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
# 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018 Python Software Foundation.
# All rights reserved.
#
# Copyright (c) 2000 BeOpen.com.  All rights reserved.
#
# Copyright (c) 1995-2001 Corporation for National Research Initiatives.  All
# rights reserved.
#
# Copyright (c) 1991-1995 Stichting Mathematisch Centrum.  All rights reserved.


from warnings import warn as _warn
from types import MethodType as _MethodType, BuiltinMethodType as _BuiltinMethodType
from math import log as _log, exp as _exp, pi as _pi, e as _e, ceil as _ceil
from math import sqrt as _sqrt, acos as _acos, cos as _cos, sin as _sin
from os import urandom as _urandom
from binascii import hexlify as _hexlify
import hashlib as _hashlib

NV_MAGICCONST = 4 * _exp(-0.5)/_sqrt(2.0)
TWOPI = 2.0*_pi
LOG4 = _log(4.0)
SG_MAGICCONST = 1.0 + _log(4.5)
BPF = 53        # Number of bits in a float

import _random

class Random(_random.Random):
    """Random number generator base class used by bound module functions.

    Used to instantiate instances of Random to get generators that don't
    share state.  Especially useful for multi-threaded programs, creating
    a different instance of Random for each thread, and using the jumpahead()
    method to ensure that the generated sequences seen by each thread don't
    overlap.

    Class Random can also be subclassed if you want to use a different basic
    generator of your own devising: in that case, override the following
    methods: random(), seed(), getstate(), setstate() and jumpahead().
    Optionally, implement a getrandbits() method so that randrange() can cover
    arbitrarily large ranges.

    """

    VERSION = 3     # used by getstate/setstate

    def __init__(self, x=None):
        """Initialize an instance.

        Optional argument x controls seeding, as for Random.seed().
        """

        self.seed(x)
        self.gauss_next = None

    def seed(self, a=None):
        """Initialize internal state of the random number generator.

        None or no argument seeds from current time or from an operating
        system specific randomness source if available.

        If a is not None or is an int or long, hash(a) is used instead.
        Hash values for some types are nondeterministic when the
        PYTHONHASHSEED environment variable is enabled.
        """

        if a is None:
            try:
                # Seed with enough bytes to span the 19937 bit
                # state space for the Mersenne Twister
                a = int(_hexlify(_urandom(2500)), 16)
            except NotImplementedError:
                import time
                a = int(time.time() * 256) # use fractional seconds

        super(Random, self).seed(a)
        self.gauss_next = None

    def getstate(self):
        """Return internal state; can be passed to setstate() later."""
        return self.VERSION, super(Random, self).getstate(), self.gauss_next

    def setstate(self, state):
        """Restore internal state from object returned by getstate()."""
        version = state[0]
        if version == 3:
            version, internalstate, self.gauss_next = state
            super(Random, self).setstate(internalstate)
        elif version == 2:
            version, internalstate, self.gauss_next = state
            # In version 2, the state was saved as signed ints, which causes
            #   inconsistencies between 32/64-bit systems. The state is
            #   really unsigned 32-bit ints, so we convert negative ints from
            #   version 2 to positive longs for version 3.
            try:
                internalstate = tuple( int(x) % (2**32) for x in internalstate )
            except ValueError as e:
                raise TypeError(e)
            super(Random, self).setstate(internalstate)
        else:
            raise ValueError("state with version %s passed to "
                             "Random.setstate() of version %s" %
                             (version, self.VERSION))

    def jumpahead(self, n):
        """Change the internal state to one that is likely far away
        from the current state.  This method will not be in Py3.x,
        so it is better to simply reseed.
        """
        # The super.jumpahead() method uses shuffling to change state,
        # so it needs a large and "interesting" n to work with.  Here,
        # we use hashing to create a large n for the shuffle.
        s = repr(n) + repr(self.getstate())
        n = int(_hashlib.new('sha512', s).hexdigest(), 16)
        super(Random, self).jumpahead(n)

## ---- Methods below this point do not need to be overridden when
## ---- subclassing for the purpose of using a different core generator.

## -------------------- pickle support  -------------------

    def __getstate__(self): # for pickle
        return self.getstate()

    def __setstate__(self, state):  # for pickle
        self.setstate(state)

    def __reduce__(self):
        return self.__class__, (), self.getstate()

## -------------------- integer methods  -------------------

    def randrange(self, start, stop=None, step=1, _int=int, _maxwidth=1<<BPF):
        """Choose a random item from range(start, stop[, step]).

        This fixes the problem with randint() which includes the
        endpoint; in Python this is usually not what you want.

        """

        # This code is a bit messy to make it fast for the
        # common case while still doing adequate error checking.
        istart = _int(start)
        if istart != start:
            raise ValueError("non-integer arg 1 for randrange()")
        if stop is None:
            if istart > 0:
                if istart >= _maxwidth:
                    return self._randbelow(istart)
                return _int(self.random() * istart)
            raise ValueError("empty range for randrange()")

        # stop argument supplied.
        istop = _int(stop)
        if istop != stop:
            raise ValueError("non-integer stop for randrange()")
        width = istop - istart
        if step == 1 and width > 0:
            # Note that
            #     int(istart + self.random()*width)
            # instead would be incorrect.  For example, consider istart
            # = -2 and istop = 0.  Then the guts would be in
            # -2.0 to 0.0 exclusive on both ends (ignoring that random()
            # might return 0.0), and because int() truncates toward 0, the
            # final result would be -1 or 0 (instead of -2 or -1).
            #     istart + int(self.random()*width)
            # would also be incorrect, for a subtler reason:  the RHS
            # can return a long, and then randrange() would also return
            # a long, but we're supposed to return an int (for backward
            # compatibility).

            if width >= _maxwidth:
                return _int(istart + self._randbelow(width))
            return _int(istart + _int(self.random()*width))
        if step == 1:
            raise ValueError("empty range for randrange() (%d,%d, %d)" %
                    (istart, istop, width))

        # Non-unit step argument supplied.
        istep = _int(step)
        if istep != step:
            raise ValueError("non-integer step for randrange()")
        if istep > 0:
            n = (width + istep - 1) // istep
        elif istep < 0:
            n = (width + istep + 1) // istep
        else:
            raise ValueError("zero step for randrange()")

        if n <= 0:
            raise ValueError("empty range for randrange()")

        if n >= _maxwidth:
            return istart + istep*self._randbelow(n)
        return istart + istep*_int(self.random() * n)

    def randint(self, a, b):
        """Return random integer in range [a, b], including both end points.
        """

        return self.randrange(a, b+1)

    def _randbelow(self, n, _log=_log, _int=int, _maxwidth=1<<BPF,
                   _Method=_MethodType, _BuiltinMethod=_BuiltinMethodType):
        """Return a random int in the range [0,n)

        Handles the case where n has more bits than returned
        by a single call to the underlying generator.
        """

        try:
            getrandbits = self.getrandbits
        except AttributeError:
            pass
        else:
            # Only call self.getrandbits if the original random() builtin method
            # has not been overridden or if a new getrandbits() was supplied.
            # This assures that the two methods correspond.
            if type(self.random) is _BuiltinMethod or type(getrandbits) is _Method:
                k = _int(1.00001 + _log(n-1, 2.0))   # 2**k > n-1 > 2**(k-2)
                r = getrandbits(k)
                while r >= n:
                    r = getrandbits(k)
                return r
        if n >= _maxwidth:
            _warn("Underlying random() generator does not supply \n"
                "enough bits to choose from a population range this large")
        return _int(self.random() * n)

## -------------------- sequence methods  -------------------

    def choice(self, seq):
        """Choose a random element from a non-empty sequence."""
        return seq[int(self.random() * len(seq))]  # raises IndexError if seq is empty

    def shuffle(self, x, random=None):
        """x, random=random.random -> shuffle list x in place; return None.

        Optional arg random is a 0-argument function returning a random
        float in [0.0, 1.0); by default, the standard random.random.

        """

        if random is None:
            random = self.random
        _int = int
        for i in reversed(range(1, len(x))):
            # pick an element in x[:i+1] with which to exchange x[i]
            j = _int(random() * (i+1))
            x[i], x[j] = x[j], x[i]

    def sample(self, population, k):
        """
        Choose k unique random elements from a population sequence.

        Return a new list containing elements from the population while
        leaving the original population unchanged.  The resulting list is
        in selection order so that all sub-slices will also be valid random
        samples.  This allows raffle winners (the sample) to be partitioned
        into grand prize and second place winners (the subslices).

        Members of the population need not be hashable or unique.  If the
        population contains repeats, then each occurrence is a possible
        selection in the sample.

        To choose a sample in a range of integers, use xrange as an argument.
        This is especially fast and space efficient for sampling from a
        large population:   sample(xrange(10000000), 60)
        """

        # Sampling without replacement entails tracking either potential
        # selections (the pool) in a list or previous selections in a set.

        # When the number of selections is small compared to the
        # population, then tracking selections is efficient, requiring
        # only a small set and an occasional reselection.  For
        # a larger number of selections, the pool tracking method is
        # preferred since the list takes less space than the
        # set and it doesn't suffer from frequent reselections.

        n = len(population)
        if not 0 <= k <= n:
            raise ValueError("sample larger than population")
        random = self.random
        _int = int
        result = [None] * k
        setsize = 21        # size of a small set minus size of an empty list
        if k > 5:
            setsize += 4 ** _ceil(_log(k * 3, 4)) # table size for big sets
        if n <= setsize or hasattr(population, "keys"):
            # An n-length list is smaller than a k-length set, or this is a
            # mapping type so the other algorithm wouldn't work.
            pool = list(population)
            for i in range(k):         # invariant:  non-selected at [0,n-i)
                j = _int(random() * (n-i))
                result[i] = pool[j]
                pool[j] = pool[n-i-1]   # move non-selected item into vacancy
        else:
            try:
                selected = set()
                selected_add = selected.add
                for i in range(k):
                    j = _int(random() * n)
                    while j in selected:
                        j = _int(random() * n)
                    selected_add(j)
                    result[i] = population[j]
            except (TypeError, KeyError):   # handle (at least) sets
                if isinstance(population, list):
                    raise
                return self.sample(tuple(population), k)
        return result

## -------------------- real-valued distributions  -------------------

## -------------------- uniform distribution -------------------

    def uniform(self, a, b):
        "Get a random number in the range [a, b) or [a, b] depending on rounding."
        return a + (b-a) * self.random()

## -------------------- triangular --------------------

    def triangular(self, low=0.0, high=1.0, mode=None):
        """Triangular distribution.

        Continuous distribution bounded by given lower and upper limits,
        and having a given mode value in-between.

        http://en.wikipedia.org/wiki/Triangular_distribution

        """
        u = self.random()
        try:
            c = 0.5 if mode is None else (mode - low) / (high - low)
        except ZeroDivisionError:
            return low
        if u > c:
            u = 1.0 - u
            c = 1.0 - c
            low, high = high, low
        return low + (high - low) * (u * c) ** 0.5

## -------------------- normal distribution --------------------

    def normalvariate(self, mu, sigma):
        """Normal distribution.

        mu is the mean, and sigma is the standard deviation.

        """
        # mu = mean, sigma = standard deviation

        # Uses Kinderman and Monahan method. Reference: Kinderman,
        # A.J. and Monahan, J.F., "Computer generation of random
        # variables using the ratio of uniform deviates", ACM Trans
        # Math Software, 3, (1977), pp257-260.

        random = self.random
        while 1:
            u1 = random()
            u2 = 1.0 - random()
            z = NV_MAGICCONST*(u1-0.5)/u2
            zz = z*z/4.0
            if zz <= -_log(u2):
                break
        return mu + z*sigma

## -------------------- lognormal distribution --------------------

    def lognormvariate(self, mu, sigma):
        """Log normal distribution.

        If you take the natural logarithm of this distribution, you'll get a
        normal distribution with mean mu and standard deviation sigma.
        mu can have any value, and sigma must be greater than zero.

        """
        return _exp(self.normalvariate(mu, sigma))

## -------------------- exponential distribution --------------------

    def expovariate(self, lambd):
        """Exponential distribution.

        lambd is 1.0 divided by the desired mean.  It should be
        nonzero.  (The parameter would be called "lambda", but that is
        a reserved word in Python.)  Returned values range from 0 to
        positive infinity if lambd is positive, and from negative
        infinity to 0 if lambd is negative.

        """
        # lambd: rate lambd = 1/mean
        # ('lambda' is a Python reserved word)

        # we use 1-random() instead of random() to preclude the
        # possibility of taking the log of zero.
        return -_log(1.0 - self.random())/lambd

## -------------------- von Mises distribution --------------------

    def vonmisesvariate(self, mu, kappa):
        """Circular data distribution.

        mu is the mean angle, expressed in radians between 0 and 2*pi, and
        kappa is the concentration parameter, which must be greater than or
        equal to zero.  If kappa is equal to zero, this distribution reduces
        to a uniform random angle over the range 0 to 2*pi.

        """
        # mu:    mean angle (in radians between 0 and 2*pi)
        # kappa: concentration parameter kappa (>= 0)
        # if kappa = 0 generate uniform random angle

        # Based upon an algorithm published in: Fisher, N.I.,
        # "Statistical Analysis of Circular Data", Cambridge
        # University Press, 1993.

        # Thanks to Magnus Kessler for a correction to the
        # implementation of step 4.

        random = self.random
        if kappa <= 1e-6:
            return TWOPI * random()

        s = 0.5 / kappa
        r = s + _sqrt(1.0 + s * s)

        while 1:
            u1 = random()
            z = _cos(_pi * u1)

            d = z / (r + z)
            u2 = random()
            if u2 < 1.0 - d * d or u2 <= (1.0 - d) * _exp(d):
                break

        q = 1.0 / r
        f = (q + z) / (1.0 + q * z)
        u3 = random()
        if u3 > 0.5:
            theta = (mu + _acos(f)) % TWOPI
        else:
            theta = (mu - _acos(f)) % TWOPI

        return theta

## -------------------- gamma distribution --------------------

    def gammavariate(self, alpha, beta):
        """Gamma distribution.  Not the gamma function!

        Conditions on the parameters are alpha > 0 and beta > 0.

        The probability distribution function is:

                    x ** (alpha - 1) * math.exp(-x / beta)
          pdf(x) =  --------------------------------------
                      math.gamma(alpha) * beta ** alpha

        """

        # alpha > 0, beta > 0, mean is alpha*beta, variance is alpha*beta**2

        # Warning: a few older sources define the gamma distribution in terms
        # of alpha > -1.0
        if alpha <= 0.0 or beta <= 0.0:
            raise ValueError('gammavariate: alpha and beta must be > 0.0')

        random = self.random
        if alpha > 1.0:

            # Uses R.C.H. Cheng, "The generation of Gamma
            # variables with non-integral shape parameters",
            # Applied Statistics, (1977), 26, No. 1, p71-74

            ainv = _sqrt(2.0 * alpha - 1.0)
            bbb = alpha - LOG4
            ccc = alpha + ainv

            while 1:
                u1 = random()
                if not 1e-7 < u1 < .9999999:
                    continue
                u2 = 1.0 - random()
                v = _log(u1/(1.0-u1))/ainv
                x = alpha*_exp(v)
                z = u1*u1*u2
                r = bbb+ccc*v-x
                if r + SG_MAGICCONST - 4.5*z >= 0.0 or r >= _log(z):
                    return x * beta

        elif alpha == 1.0:
            # expovariate(1)
            u = random()
            while u <= 1e-7:
                u = random()
            return -_log(u) * beta

        else:   # alpha is between 0 and 1 (exclusive)

            # Uses ALGORITHM GS of Statistical Computing - Kennedy & Gentle

            while 1:
                u = random()
                b = (_e + alpha)/_e
                p = b*u
                if p <= 1.0:
                    x = p ** (1.0/alpha)
                else:
                    x = -_log((b-p)/alpha)
                u1 = random()
                if p > 1.0:
                    if u1 <= x ** (alpha - 1.0):
                        break
                elif u1 <= _exp(-x):
                    break
            return x * beta

## -------------------- Gauss (faster alternative) --------------------

    def gauss(self, mu, sigma):
        """Gaussian distribution.

        mu is the mean, and sigma is the standard deviation.  This is
        slightly faster than the normalvariate() function.

        Not thread-safe without a lock around calls.

        """

        # When x and y are two variables from [0, 1), uniformly
        # distributed, then
        #
        #    cos(2*pi*x)*sqrt(-2*log(1-y))
        #    sin(2*pi*x)*sqrt(-2*log(1-y))
        #
        # are two *independent* variables with normal distribution
        # (mu = 0, sigma = 1).
        # (Lambert Meertens)
        # (corrected version; bug discovered by Mike Miller, fixed by LM)

        # Multithreading note: When two threads call this function
        # simultaneously, it is possible that they will receive the
        # same return value.  The window is very small though.  To
        # avoid this, you have to use a lock around all calls.  (I
        # didn't want to slow this down in the serial case by using a
        # lock here.)

        random = self.random
        z = self.gauss_next
        self.gauss_next = None
        if z is None:
            x2pi = random() * TWOPI
            g2rad = _sqrt(-2.0 * _log(1.0 - random()))
            z = _cos(x2pi) * g2rad
            self.gauss_next = _sin(x2pi) * g2rad

        return mu + z*sigma

## -------------------- beta --------------------
## See
## http://mail.python.org/pipermail/python-bugs-list/2001-January/003752.html
## for Ivan Frohne's insightful analysis of why the original implementation:
##
##    def betavariate(self, alpha, beta):
##        # Discrete Event Simulation in C, pp 87-88.
##
##        y = self.expovariate(alpha)
##        z = self.expovariate(1.0/beta)
##        return z/(y+z)
##
## was dead wrong, and how it probably got that way.

    def betavariate(self, alpha, beta):
        """Beta distribution.

        Conditions on the parameters are alpha > 0 and beta > 0.
        Returned values range between 0 and 1.

        """

        # This version due to Janne Sinkkonen, and matches all the std
        # texts (e.g., Knuth Vol 2 Ed 3 pg 134 "the beta distribution").
        y = self.gammavariate(alpha, 1.)
        if y == 0:
            return 0.0
        else:
            return y / (y + self.gammavariate(beta, 1.))

## -------------------- Pareto --------------------

    def paretovariate(self, alpha):
        """Pareto distribution.  alpha is the shape parameter."""
        # Jain, pg. 495

        u = 1.0 - self.random()
        return 1.0 / pow(u, 1.0/alpha)

## -------------------- Weibull --------------------

    def weibullvariate(self, alpha, beta):
        """Weibull distribution.

        alpha is the scale parameter and beta is the shape parameter.

        """
        # Jain, pg. 499; bug fix courtesy Bill Arms

        u = 1.0 - self.random()
        return alpha * pow(-_log(u), 1.0/beta)



# -*- coding: utf-8 -*-
"""
Performance Test for Clone Protocol

see :class:`sage.structure.list_clone.ClonableArray`

EXAMPLES::

    sage: from sage.structure.list_clone_timings import *
    sage: cmd =["",
    ....:     "e.__copy__()",
    ....:     "copy(e)",
    ....:     "e.clone()",
    ....:     "e.__class__(e.parent(), e._get_list())",
    ....:     "e.__class__(e.parent(), e[:])",
    ....:     "e.check()",
    ....:     "",
    ....:     "add1_internal(e)",
    ....:     "add1_immutable(e)",
    ....:     "add1_mutable(e)",
    ....:     "add1_with(e)",
    ....:     "",
    ....:     "cy_add1_internal(e)",
    ....:     "cy_add1_immutable(e)",
    ....:     "cy_add1_mutable(e)",
    ....:     "cy_add1_with(e)"]

Various timings using a Cython class::

    sage: size = 5
    sage: e = IncreasingArrays()(range(size))
    sage: # random
    ....: for p in cmd:
    ....:     print("{0:36} : ".format(p), end=""); timeit(p)
                                         :
    e.__copy__()                         :  625 loops, best of 3: 446 ns per loop
    copy(e)                              :  625 loops, best of 3: 1.94 µs per loop
    e.clone()                            :  625 loops, best of 3: 736 ns per loop
    e.__class__(e.parent(), e._get_list()) :  625 loops, best of 3: 1.34 µs per loop
    e.__class__(e.parent(), e[:])        :  625 loops, best of 3: 1.35 µs per loop
    e.check()                            :  625 loops, best of 3: 342 ns per loop
                                         :
    add1_internal(e)                     :  625 loops, best of 3: 3.53 µs per loop
    add1_immutable(e)                    :  625 loops, best of 3: 3.72 µs per loop
    add1_mutable(e)                      :  625 loops, best of 3: 3.42 µs per loop
    add1_with(e)                         :  625 loops, best of 3: 4.05 µs per loop
                                         :
    cy_add1_internal(e)                  :  625 loops, best of 3: 752 ns per loop
    cy_add1_immutable(e)                 :  625 loops, best of 3: 1.28 µs per loop
    cy_add1_mutable(e)                   :  625 loops, best of 3: 861 ns per loop
    cy_add1_with(e)                      :  625 loops, best of 3: 1.51 µs per loop

Various timings using a Python class::

    sage: e = IncreasingArraysPy()(range(size))
    sage: # random
    ....: for p in cmd: print("{0:36} : ".format(p), end=""); timeit(p)
                                         :
    e.__copy__()                         :  625 loops, best of 3: 869 ns per loop
    copy(e)                              :  625 loops, best of 3: 2.13 µs per loop
    e.clone()                            :  625 loops, best of 3: 1.86 µs per loop
    e.__class__(e.parent(), e._get_list()) :  625 loops, best of 3: 7.52 µs per loop
    e.__class__(e.parent(), e[:])        :  625 loops, best of 3: 7.27 µs per loop
    e.check()                            :  625 loops, best of 3: 4.02 µs per loop
                                         :
    add1_internal(e)                     :  625 loops, best of 3: 9.34 µs per loop
    add1_immutable(e)                    :  625 loops, best of 3: 9.91 µs per loop
    add1_mutable(e)                      :  625 loops, best of 3: 12.6 µs per loop
    add1_with(e)                         :  625 loops, best of 3: 15.9 µs per loop
                                         :
    cy_add1_internal(e)                  :  625 loops, best of 3: 7.13 µs per loop
    cy_add1_immutable(e)                 :  625 loops, best of 3: 6.95 µs per loop
    cy_add1_mutable(e)                   :  625 loops, best of 3: 14.1 µs per loop
    cy_add1_with(e)                      :  625 loops, best of 3: 17.5 µs per loop
"""
#*****************************************************************************
#  Copyright (C) 2009-2010 Florent Hivert <Florent.Hivert@univ-rouen.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.structure.list_clone import ClonableArray
from sage.structure.list_clone_demo import IncreasingArrays


class IncreasingArraysPy(IncreasingArrays):

    class Element(ClonableArray):
        """
        A small class for testing :class:`ClonableArray`: Increasing Lists

        TESTS::

            sage: from sage.structure.list_clone_timings import IncreasingArraysPy
            sage: TestSuite(IncreasingArraysPy()([1,2,3])).run()
        """

        def check(self):
            """
            Check that ``self`` is increasing.

            EXAMPLES::

                sage: from sage.structure.list_clone_timings import IncreasingArraysPy
                sage: IncreasingArraysPy()([1,2,3]) # indirect doctest
                [1, 2, 3]
                sage: IncreasingArraysPy()([3,2,1]) # indirect doctest
                Traceback (most recent call last):
                ...
                ValueError: Lists is not increasing
            """
            for i in range(len(self)-1):
                if self[i] >= self[i+1]:
                    raise ValueError("Lists is not increasing")


#####################################################################
######                    Timings functions                    ######
#####################################################################
def add1_internal(bla):
    """
    TESTS::

        sage: from sage.structure.list_clone_timings import *
        sage: add1_internal(IncreasingArrays()([1,4,5]))
        [2, 5, 6]
    """
    blo = bla.__copy__()
    lst = blo._get_list()
    for i in range(len(blo)):
        lst[i] += 1
    blo.set_immutable()
    blo.check()
    return blo

def add1_immutable(bla):
    """
    TESTS::

        sage: from sage.structure.list_clone_timings import *
        sage: add1_immutable(IncreasingArrays()([1,4,5]))
        [2, 5, 6]
    """
    lbla = bla[:]
    for i in range(len(lbla)):
        lbla[i] += 1
    return bla.__class__(bla.parent(), lbla)

def add1_mutable(bla):
    """
    TESTS::

        sage: from sage.structure.list_clone_timings import *
        sage: add1_mutable(IncreasingArrays()([1,4,5]))
        [2, 5, 6]
    """
    blo = bla.__copy__()
    for i in range(len(blo)):
        blo[i] += 1
    blo.set_immutable()
    blo.check()
    return blo

def add1_with(bla):
    """
    TESTS::

        sage: from sage.structure.list_clone_timings import *
        sage: add1_with(IncreasingArrays()([1,4,5]))
        [2, 5, 6]
    """
    with bla.clone() as blo:
        for i in range(len(blo)):
            blo[i] += 1
    return blo



"""
Formal sums

AUTHORS:

 - David Harvey (2006-09-20): changed FormalSum not to derive from
   "list" anymore, because that breaks new Element interface

 - Nick Alexander (2006-12-06): added test cases.

 - William Stein (2006, 2009): wrote the first version in 2006, documented it in 2009.

 - Volker Braun (2010-07-19): new-style coercions, documentation
   added. FormalSums now derives from UniqueRepresentation.

FUNCTIONS:
    - ``FormalSums(ring)`` -- create the module of formal finite sums with
                          coefficients in the given ring.

    - ``FormalSum(list of pairs (coeff, number))`` -- create a formal sum

EXAMPLES::

    sage: A = FormalSum([(1, 2/3)]); A
    2/3
    sage: B = FormalSum([(3, 1/5)]); B
    3*1/5
    sage: -B
    -3*1/5
    sage: A + B
    2/3 + 3*1/5
    sage: A - B
    2/3 - 3*1/5
    sage: B*3
    9*1/5
    sage: 2*A
    2*2/3
    sage: list(2*A + A)
    [(3, 2/3)]

TESTS::

    sage: R = FormalSums(QQ)
    sage: loads(dumps(R)) == R
    True
    sage: a = R(2/3) + R(-5/7); a
    2/3 + -5/7
    sage: loads(dumps(a)) == a
    True
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.repr import repr_lincomb
import operator
from collections import OrderedDict

from sage.modules.module import Module
from sage.structure.element import ModuleElement
from sage.structure.richcmp import richcmp
from sage.rings.integer_ring import ZZ
from sage.structure.parent import Parent
from sage.structure.coerce import LeftModuleAction, RightModuleAction
from sage.categories.action import PrecomposedAction
from sage.structure.unique_representation import UniqueRepresentation


class FormalSum(ModuleElement):
    """
    A formal sum over a ring.
    """
    def __init__(self, x, parent=None, check=True, reduce=True):
        """
        INPUT:
            - ``x`` -- object
            - ``parent`` -- FormalSums(R) module (default: FormalSums(ZZ))
            - ``check`` -- bool (default: True) if False, might not coerce
                           coefficients into base ring, which can speed
                           up constructing a formal sum.
            - ``reduce`` -- reduce (default: True) if False, do not
                            combine common terms

        EXAMPLES::

            sage: FormalSum([(1,2/3), (3,2/3), (-5, 7)])
            4*2/3 - 5*7
            sage: a = FormalSum([(1,2/3), (3,2/3), (-5, 7)], reduce=False); a
            2/3 + 3*2/3 - 5*7
            sage: a.reduce()
            sage: a
            4*2/3 - 5*7
            sage: FormalSum([(1,2/3), (3,2/3), (-5, 7)], parent=FormalSums(GF(5)))
            4*2/3

        Notice below that the coefficient 5 doesn't get reduced modulo 5::

            sage: FormalSum([(1,2/3), (3,2/3), (-5, 7)], parent=FormalSums(GF(5)), check=False)
            4*2/3 - 5*7

        Make sure we first reduce before checking coefficient types::

            sage: x,y = var('x, y')
            sage: FormalSum([(1/2,x), (2,y)], FormalSums(QQ))
            1/2*x + 2*y
            sage: FormalSum([(1/2,x), (2,y)], FormalSums(ZZ))
            Traceback (most recent call last):
            ...
            TypeError: no conversion of this rational to integer
            sage: FormalSum([(1/2,x), (1/2,x), (2,y)], FormalSums(ZZ))
            x + 2*y
        """
        if x == 0:
            x = []
        self._data = x
        if parent is None:
            parent = formal_sums
        ModuleElement.__init__(self, parent)
        assert isinstance(parent, parent.category().parent_class)
        if reduce:  # first reduce
            self.reduce()
        if check:   # then check
            k = parent.base_ring()
            try:
                self._data = [(k(t[0]), t[1]) for t in self._data]
            except (IndexError, KeyError) as msg:
                raise TypeError("%s\nInvalid formal sum"%msg)

    def __iter__(self):
        """
        EXAMPLES::

            sage: for z in FormalSum([(1, 2), (-3, 7), (5, 1000)]):
            ....:     print(z)
            (1, 2)
            (-3, 7)
            (5, 1000)
        """
        return iter(self._data)

    def __getitem__(self, n):
        """
        EXAMPLES::

            sage: v = FormalSum([(1, 2), (-3, 7), (5, 1000)]); v
            2 - 3*7 + 5*1000
            sage: v[0]
            (1, 2)
            sage: v[1]
            (-3, 7)
            sage: v[2]
            (5, 1000)
            sage: list(v)
            [(1, 2), (-3, 7), (5, 1000)]
        """
        return self._data[n]

    def __len__(self):
        """
        EXAMPLES::

            sage: v = FormalSum([(1, 2), (5, 1000), (-3, 7)])
            sage: len(v)
            3
        """
        return len(self._data)

    def _repr_(self):
        """
        EXAMPLES::

            sage: a = FormalSum([(1,2/3), (-3,4/5), (7,Mod(2,3))])
            sage: a   # random, since comparing Mod(2,3) and rationals ill-defined
            sage: a._repr_()    # random
            '2/3 - 3*4/5 + 7*2'
        """
        return repr_lincomb([t, c] for c, t in self)

    def _latex_(self):
        r"""
        EXAMPLES::

            sage: latex(FormalSum([(1,2), (5, 8/9), (-3, 7)]))
            2 + 5\cdot \frac{8}{9} - 3\cdot 7
        """
        from sage.misc.latex import repr_lincomb
        symbols = [z[1] for z in self]
        coeffs = [z[0] for z in self]
        return repr_lincomb(symbols, coeffs)
        # TODO: finish merging sage.misc.latex.repr_lincomb and
        # sage.misc.misc.repr_lincomb and use instead:
        # return repr_lincomb([[t,c] for c,t in self], is_latex=True)

    def _richcmp_(self, other, op):
        """
        Compare ``self`` and ``other``.

        INPUT:

        - ``other`` -- a :class:`FormalSum` with the same parent

        - ``op`` -- a comparison operator

        EXAMPLES::

            sage: a = FormalSum([(1,3),(2,5)]); a
            3 + 2*5
            sage: b = FormalSum([(1,3),(2,7)]); b
            3 + 2*7
            sage: a != b
            True
            sage: a_QQ = FormalSum([(1,3),(2,5)],parent=FormalSums(QQ))
            sage: a == a_QQ       # a is coerced into FormalSums(QQ)
            True
            sage: a == 0          # 0 is coerced into a.parent()(0)
            False
        """
        return richcmp(self._data, other._data, op)

    def _neg_(self):
        """
        EXAMPLES::

            sage: -FormalSum([(1,3),(2,5)])
            -3 - 2*5
        """
        return self.__class__([(-c, s) for (c, s) in self._data], check=False, parent=self.parent())

    def _add_(self, other):
        """
        EXAMPLES::

            sage: FormalSum([(1,3/7),(2,5/8)]) + FormalSum([(1,3/7),(-2,5)])  # indirect doctest
            2*3/7 + 2*5/8 - 2*5
        """
        return self.__class__(self._data + other._data, check=False, parent=self.parent())

    def _lmul_(self, s):
        """
        EXAMPLES::

            sage: FormalSum([(1,3/7),(-2,5)])*(-3)
            -3*3/7 + 6*5
        """
        return self.__class__([(c*s, x) for (c, x) in self], check=False, parent=self.parent())

    def _rmul_(self, s):
        """
        EXAMPLES::

            sage: -3*FormalSum([(1,3/7),(-2,5)])
            -3*3/7 + 6*5
        """
        return self.__class__([(s*c, x) for (c, x) in self], check=False, parent=self.parent())

    def __bool__(self):
        """
        EXAMPLES::

            sage: bool(FormalSum([(1,3/7),(-2,5)]))
            True
            sage: bool(FormalSums(QQ)(0))
            False
            sage: bool(FormalSums(QQ)(1))
            True
        """
        if not len(self._data):
            return False
        for c, _ in self._data:
            if not c.is_zero():
                return True
        return False

    __nonzero__ = __bool__

    def reduce(self):
        """
        EXAMPLES::

            sage: a = FormalSum([(-2,3), (2,3)], reduce=False); a
            -2*3 + 2*3
            sage: a.reduce()
            sage: a
            0
        """
        new = OrderedDict()
        for coeff, x in self:
            try:
                coeff += new[x]
            except KeyError:
                pass
            new[x] = coeff
        self._data = [(c, x) for (x, c) in new.items() if c]


class FormalSums(UniqueRepresentation, Module):
    """
    The R-module of finite formal sums with coefficients in some ring R.

    EXAMPLES::

        sage: FormalSums()
        Abelian Group of all Formal Finite Sums over Integer Ring
        sage: FormalSums(ZZ)
        Abelian Group of all Formal Finite Sums over Integer Ring
        sage: FormalSums(GF(7))
        Abelian Group of all Formal Finite Sums over Finite Field of size 7
        sage: FormalSums(ZZ[sqrt(2)])
        Abelian Group of all Formal Finite Sums over Order in Number Field in sqrt2 with defining polynomial x^2 - 2 with sqrt2 = 1.414213562373095?
        sage: FormalSums(GF(9,'a'))
        Abelian Group of all Formal Finite Sums over Finite Field in a of size 3^2

    TESTS::

        sage: TestSuite(FormalSums(QQ)).run()

    """
    Element = FormalSum
    @staticmethod
    def __classcall__(cls, base_ring = ZZ):
        """
        Set the default value for the base ring.

        EXAMPLES::

            sage: FormalSums(ZZ) == FormalSums()    # indirect test
            True
        """
        return UniqueRepresentation.__classcall__(cls, base_ring)

    def _repr_(self):
        """
        EXAMPLES::

            sage: FormalSums(GF(7))
            Abelian Group of all Formal Finite Sums over Finite Field of size 7
            sage: FormalSums(GF(7))._repr_()
            'Abelian Group of all Formal Finite Sums over Finite Field of size 7'
        """
        return "Abelian Group of all Formal Finite Sums over %s"%self.base_ring()

    def _element_constructor_(self, x, check=True, reduce=True):
        """
        Make a formal sum in self from x.

        INPUT:

        - ``x`` -- formal sum, list or number

        - ``check`` -- bool (default: True)

        - ``reduce`` -- bool (default: True); whether to combine terms

        EXAMPLES::

            sage: P = FormalSum([(1,2/3)]).parent()
            sage: P([(1,2/3), (5,-2/9)])  # indirect test
            2/3 + 5*-2/9
        """
        if isinstance(x, FormalSum):
            P = x.parent()
            if P is self:
                return x
            else:
                x = x._data
        if isinstance(x, list):
            return self.element_class(x, check=check,reduce=reduce,parent=self)
        if x == 0:
            return self.element_class([], check=False, reduce=False, parent=self)
        else:
            return self.element_class([(self.base_ring()(1), x)], check=False, reduce=False, parent=self)

    def _coerce_map_from_(self, X):
        r"""
        Return whether there is a coercion from ``X``

        EXAMPLES::

            sage: FormalSums(QQ).has_coerce_map_from( FormalSums(ZZ) )   # indirect test
            True

            sage: FormalSums(ZZ).get_action(QQ)   # indirect test
            Right scalar multiplication by Rational Field on Abelian Group of all Formal Finite Sums over Rational Field
            with precomposition on left by Coercion map:
              From: Abelian Group of all Formal Finite Sums over Integer Ring
              To:   Abelian Group of all Formal Finite Sums over Rational Field
        """
        if isinstance(X,FormalSums):
            if self.base_ring().has_coerce_map_from(X.base_ring()):
                return True
        return False

    def base_extend(self, R):
        """
        EXAMPLES::

            sage: F7 = FormalSums(ZZ).base_extend(GF(7)); F7
            Abelian Group of all Formal Finite Sums over Finite Field of size 7

        The following tests against a bug that was fixed at :trac:`18795`::

            sage: isinstance(F7, F7.category().parent_class)
            True
        """
        if self.base_ring().has_coerce_map_from(R):
            return self
        elif R.has_coerce_map_from(self.base_ring()):
            return FormalSums(R)

    def _get_action_(self, other, op, self_is_left):
        """
        EXAMPLES::

            sage: A = FormalSums(RR);  A.get_action(RR)     # indirect doctest
            Right scalar multiplication by Real Field with 53 bits of precision on Abelian Group of all Formal Finite Sums over Real Field with 53 bits of precision

            sage: A = FormalSums(ZZ);  A.get_action(QQ)
            Right scalar multiplication by Rational Field on Abelian Group of all Formal Finite Sums over Rational Field
            with precomposition on left by Coercion map:
              From: Abelian Group of all Formal Finite Sums over Integer Ring
              To:   Abelian Group of all Formal Finite Sums over Rational Field
            sage: A = FormalSums(QQ);  A.get_action(ZZ)
            Right scalar multiplication by Integer Ring on Abelian Group of all Formal Finite Sums over Rational Field
        """
        if op is operator.mul and isinstance(other, Parent):
            extended = self.base_extend(other)
            if self_is_left:
                action = RightModuleAction(other, extended)
                if extended is not self:
                    action = PrecomposedAction(action, extended._internal_coerce_map_from(self), None)
            else:
                action = LeftModuleAction(other, extended)
                if extended is not self:
                    action = PrecomposedAction(action, None, extended._internal_coerce_map_from(self))
            return action

    def _an_element_(self, check=False, reduce=False):
        """
        EXAMPLES::

            sage: FormalSums(ZZ).an_element()     # indirect test
            1
            sage: FormalSums(QQ).an_element()
            1/2*1
            sage: QQ.an_element()
            1/2
        """
        return self.element_class([(self.base_ring().an_element(), 1)],
                         check=check, reduce=reduce, parent=self)


formal_sums = FormalSums()

# Formal sums now derives from UniqueRepresentation, which makes the
# factory function unnecessary. This is why the name was changed from
# class FormalSums_generic to class FormalSums.
from sage.misc.persist import register_unpickle_override
register_unpickle_override('sage.structure.formal_sum', 'FormalSums_generic', FormalSums)



cdef extern from "Python.h":
    cdef size_t SIZEOF_VOID_P

cdef class FastHashable_class:
    cdef Py_ssize_t _hash

cdef inline long hash_by_id(void * p):
    r"""
    This function is a copy paste from the default Python hash function.
    """
    cdef long x
    cdef size_t y = <size_t>p
    # bottom 3 or 4 bits are likely to be 0; rotate y by 4 to avoid
    # excessive hash collisions for dicts and sets
    y = (y >> 4) | (y << (8 * SIZEOF_VOID_P - 4))
    x = <long>y
    if x == -1:
        x = -2
    return x



cdef class ArgumentFixer:
    cdef public object f
    cdef public int _ndefault
    cdef public int _nargs
    cdef tuple _arg_names
    cdef bint _classmethod
    cdef dict _defaults
    cdef public tuple _default_tuple

    cdef fix_to_pos_args_kwds(self, tuple args, dict kwargs)



r"""
Super Algebras
"""
#*****************************************************************************
#  Copyright (C) 2015 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.super_modules import SuperModulesCategory
from sage.categories.signed_tensor import SignedTensorProductsCategory, tensor_signed
from sage.categories.tensor import tensor
from sage.misc.lazy_import import LazyImport
from sage.misc.cachefunc import cached_method

class SuperAlgebras(SuperModulesCategory):
    r"""
    The category of super algebras.

    An `R`-*super algebra* is an `R`-super module `A` endowed with an
    `R`-algebra structure satisfying

    .. MATH::

        A_0 A_0 \subseteq A_0, \qquad
        A_0 A_1 \subseteq A_1, \qquad
        A_1 A_0 \subseteq A_1, \qquad
        A_1 A_1 \subseteq A_0

    and `1 \in A_0`.

    EXAMPLES::

        sage: Algebras(ZZ).Super()
        Category of super algebras over Integer Ring

    TESTS::

        sage: TestSuite(Algebras(ZZ).Super()).run()
    """
    def extra_super_categories(self):
        """
        EXAMPLES::

            sage: Algebras(ZZ).Super().super_categories() # indirect doctest
            [Category of graded algebras over Integer Ring,
             Category of super modules over Integer Ring]
        """
        return [self.base_category().Graded()]

    Supercommutative = LazyImport('sage.categories.supercommutative_algebras',
                                  'SupercommutativeAlgebras')

    class ParentMethods:
        def graded_algebra(self):
            r"""
            Return the associated graded algebra to ``self``.

            .. WARNING::

                Because a super module `M` is naturally `\ZZ / 2 \ZZ`-graded, and
                graded modules have a natural filtration induced by the grading, if
                `M` has a different filtration, then the associated graded module
                `\operatorname{gr} M \neq M`. This is most apparent with super
                algebras, such as the :class:`differential Weyl algebra
                <sage.algebras.weyl_algebra.DifferentialWeylAlgebra>`, and the
                multiplication may not coincide.
            """
            raise NotImplementedError

        def tensor(*parents, **kwargs):
            """
            Return the tensor product of the parents.

            EXAMPLES::

                sage: A.<x,y,z> = ExteriorAlgebra(ZZ); A.rename("A")
                sage: T = A.tensor(A,A); T
                A # A # A
                sage: T in Algebras(ZZ).Graded().SignedTensorProducts()
                True
                sage: T in Algebras(ZZ).Graded().TensorProducts()
                False
                sage: A.rename(None)

            This also works when the other elements do not have
            a signed tensor product (:trac:`31266`)::

                sage: a = SteenrodAlgebra(3).an_element()
                sage: M = CombinatorialFreeModule(GF(3), ['s', 't', 'u'])
                sage: s = M.basis()['s']
                sage: tensor([a, s])
                2*Q_1 Q_3 P(2,1) # B['s']
            """
            constructor = kwargs.pop('constructor', tensor_signed)
            try:
                cat = constructor.category_from_parents(parents)
            except AttributeError:
                # Fall back to the usual tensor product if the other parents
                #   do not support signed tensor products
                cat = tensor.category_from_parents(parents)
            return parents[0].__class__.Tensor(parents, category=cat)

    class SubcategoryMethods:
        @cached_method
        def Supercommutative(self):
            r"""
            Return the full subcategory of the supercommutative objects
            of ``self``.

            A super algebra `M` is *supercommutative* if, for all
            homogeneous `x,y\in M`,

            .. MATH::

                x \cdot y = (-1)^{|x||y|} y \cdot x.

            REFERENCES:

            :wikipedia:`Supercommutative_algebra`

            EXAMPLES::

                sage: Algebras(ZZ).Super().Supercommutative()
                Category of supercommutative algebras over Integer Ring
                sage: Algebras(ZZ).Super().WithBasis().Supercommutative()
                Category of supercommutative algebras with basis over Integer Ring
            """
            return self._with_axiom('Supercommutative')

    class SignedTensorProducts(SignedTensorProductsCategory):
        @cached_method
        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: Coalgebras(QQ).Graded().SignedTensorProducts().extra_super_categories()
                [Category of graded coalgebras over Rational Field]
                sage: Coalgebras(QQ).Graded().SignedTensorProducts().super_categories()
                [Category of graded coalgebras over Rational Field]

            Meaning: a signed tensor product of coalgebras is a coalgebra
            """
            return [self.base_category()]




"""
Various functions to deal with conversion mpz <-> Python int/long

For doctests, see :class:`Integer`.

AUTHORS:

- Gonzalo Tornaria (2006): initial version

- David Harvey (2007-08-18): added ``mpz_get_pyintlong`` function
  (:trac:`440`)

- Jeroen Demeyer (2015-02-24): moved from c_lib, rewritten using
  ``mpz_export`` and ``mpz_import`` (:trac:`17853`)
"""

#*****************************************************************************
#       Copyright (C) 2015 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************


from cpython.object cimport Py_SIZE
from cpython.int cimport PyInt_FromLong
from cpython.long cimport PyLong_FromLong
from cpython.longintrepr cimport _PyLong_New, py_long, digit, PyLong_SHIFT
from .mpz cimport *

cdef extern from *:
    Py_ssize_t* Py_SIZE_PTR "&Py_SIZE"(object)
    int hash_bits """
        #ifdef _PyHASH_BITS
        _PyHASH_BITS         /* Python 3 */
        #else
        (8 * sizeof(void*))  /* Python 2 */
        #endif
        """
    int limb_bits "(8 * sizeof(mp_limb_t))"


# Unused bits in every PyLong digit
cdef size_t PyLong_nails = 8*sizeof(digit) - PyLong_SHIFT


cdef mpz_get_pylong_large(mpz_srcptr z):
    """
    Convert a non-zero ``mpz`` to a Python ``long``.
    """
    cdef size_t nbits = mpz_sizeinbase(z, 2)
    cdef size_t pylong_size = (nbits + PyLong_SHIFT - 1) // PyLong_SHIFT
    L = _PyLong_New(pylong_size)
    mpz_export(L.ob_digit, NULL,
            -1, sizeof(digit), 0, PyLong_nails, z)
    if mpz_sgn(z) < 0:
        # Set correct size (use a pointer to hack around Cython's
        # non-support for lvalues).
        sizeptr = Py_SIZE_PTR(L)
        sizeptr[0] = -pylong_size
    return L


cdef mpz_get_pylong(mpz_srcptr z):
    """
    Convert an ``mpz`` to a Python ``long``.
    """
    if mpz_fits_slong_p(z):
        return PyLong_FromLong(mpz_get_si(z))
    return mpz_get_pylong_large(z)


cdef mpz_get_pyintlong(mpz_srcptr z):
    """
    Convert an ``mpz`` to a Python ``int`` if possible, or a ``long``
    if the value is too large.
    """
    if mpz_fits_slong_p(z):
        return PyInt_FromLong(mpz_get_si(z))
    return mpz_get_pylong_large(z)


cdef int mpz_set_pylong(mpz_ptr z, L) except -1:
    """
    Convert a Python ``long`` `L` to an ``mpz``.
    """
    cdef Py_ssize_t pylong_size = Py_SIZE(L)
    if pylong_size < 0:
        pylong_size = -pylong_size
    mpz_import(z, pylong_size, -1, sizeof(digit), 0, PyLong_nails,
            (<py_long>L).ob_digit)
    if Py_SIZE(L) < 0:
        mpz_neg(z, z)


cdef Py_hash_t mpz_pythonhash(mpz_srcptr z):
    """
    Hash an ``mpz``, where the hash value is the same as the hash value
    of the corresponding Python ``int`` or ``long``, except that we do
    not replace -1 by -2 (the Cython wrapper for ``__hash__`` does that).
    """
    if mpz_sgn(z) == 0:
        return 0

    # The hash value equals z % m where m = 2 ^ hash_bits - 1.
    #
    # Safely compute 2 ^ hash_bits - 1 without overflow
    cdef mp_limb_t modulus = (((<mp_limb_t>(1) << (hash_bits - 1)) - 1) * 2) + 1

    cdef mp_limb_t h = 0
    cdef mp_limb_t x, y
    cdef size_t i, n
    cdef unsigned int r
    n = mpz_size(z)
    for i in range(n):
        x = mpz_getlimbn(z, i)

        # Computing modulo 2 ^ hash_bits - 1 means that the bit at
        # position j is really moved to position (j % hash_bits).
        # We need to shift every bit of x left by (limb_bits * i)
        # and then put it in the right position to account for
        # the modulo operation. Store the result in y.
        if limb_bits == hash_bits:
            y = x
        else:
            r = (limb_bits * i) % hash_bits
            y = (x << r) & modulus
            y += (x >> (hash_bits - r)) & modulus
            # Only do this shift if we don't shift more than the size of the
            # type
            if r > 2 * hash_bits - limb_bits:
                y += (x >> (2 * hash_bits - r))
            # At this point, y <= 2 * modulus, so y did not overflow, but we
            # need y <= modulus. We use > instead of >= on the line below
            # because it generates more efficient code.
            if y > modulus:
                y -= modulus

        # Safely compute h = (h + y) % modulus knowing that h < modulus
        # and y <= modulus
        if h < modulus - y:
            h = h + y
        else:
            h = h - (modulus - y)

    # Special case for Python 2
    if limb_bits == hash_bits and h == 0:
        h = -1

    if mpz_sgn(z) < 0:
        return -h
    return h


assert hash_bits <= limb_bits <= 2 * hash_bits
